#!/usr/bin/perl
# This script extracts CDS, pseudogenes and RNAs from GenBank formatted seq files.


#
#    Copyright 2010 Ole Skovgaard.
#    The scripts are distributed under the terms of the GNU General Public License
#
#
#    This file is part of R2R - From Read to Results.
#
#    R2R is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, version 3 of the License.
#
#    R2R is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with R2R.  If not, see <http://www.gnu.org/licenses/>.
#
#    Version: Release_0.950, November, 2011
#


tid();  # The Danish tid is used instead of time, to confuse UltraEdit
$time_start = $tid;

if ( $^O =~ /Win/) { 	$platform = "DOS" } else  { $platform = "Linux/Unix" } # System?
print "\n$tid: Start   System is: $platform \n";

$fid=$ARGV[0];
unless (open(in,"<$fid"))  {
	print "GenBank formatted file to be analyzed \n\n" ;
	$fid=<stdin>;
	chomp $fid;
	open(in,"<$fid");
}
$_ = <in>;
unless ($_ =~ /LOCUS/ ) {
	print "First line of $fid is:\n$_\n - are you sure that $fid is a GenBank formatted file? yes/no:\n";
	my $rc = <stdin>;
	unless ($rc =~ /y/i ) {print "Try again with the correct GenBank file\n"; exit}
}
close in;
print "\n$tid  Annotations in $fid will be extracted \n\n" ;

open(GenBank,"<$fid");
open(CDS,">$fid.cds.temp");
open(SEQ,">$fid.fa");
# open(report,">$fid_annotation_report.txt");

$features ="";
$screen = select (); # Saves the value for default output (to the screen), used after flushing of output files.
binmode out;   # Ensures that System independent Unix "LF" (0A) is used for New Line.

while (<GenBank>) {
	if ($_ =~ /FEATURES/ ) {$features ="FEATURE"}
	if ($_ =~ /^ORIGIN/ ) {$features ="SEQUENCE"}

	if ($features == "FEATURE" ) {  # delimit of FEATURES
		$a20 = substr($_, 0, 20);
		if ( $a20 =~ /\s{5}(\S+)\s{5}/ ) {  # this action when feature changes
			#empty previous data, start acq new data:
			#read data untill a new $ann_type_n is met, then analyze with extanno ();
			$ann_type_n = $1;
			if ($ann_type =~ /CDS/ || $ann_type =~ /rna/i ){
				extanno ();  # extract and analyze
			} # end if
			$ann_type = $ann_type_n;
			@anno =();
			push (@anno, $_); #Start the next annotation
	
		} #endif $a20
		else {
			push (@anno, $_); # Continue to push the next annotation
		}
	}  # endif features
	
	tid();  # The Danish tid is used instead of time, to confuse UltraEdit
	$time_fe = $tid;
	
	if ($_ =~ /^LOCUS/) {$loc = substr( $_, 12, length($_))}
	if ($features =~/SEQUENCE/ ) {  # delimit of sequence
		s/\s//g;   #substitute any whitespace
		s/\d//g;   #substitute any digit
		$_ .= "\n";    #add end of line
		if ($_ =~ /ORIGIN/ ) {
			$_ = ">".$loc;
		}
		print SEQ "$_";  # and print to file
	}
}  # end while GenBank

# Flush SEQ;
select (SEQ);  $| = 1;  $| = 0;
select (CDS);  $| = 1;  $| = 0;
select ($screen);
close SEQ;
close CDS;

tid();
$sort_in  = "$fid.cds.temp";
$sort_out = "$fid.cds";
print "\n\n$tid Sorting annotations in $sort_in to $sort_out \n";
if ($platform eq "DOS" ) {
	system ("sort $sort_in /O $sort_out /REC 65535");
}
else {
	system ("sort -n $sort_in > $sort_out");
}
tid();
print "$tid Sorting $sort_in to $sort_out finished\n\n";

if ($platform eq "DOS" ) { #deletes temporary files;
	system ("del *.temp");
}
else {
	system ("rm *.temp");
}

tid();  # The Danish tid is used instead of time, to confuse UltraEdit
$time_end = $tid;

print "
Analysis started at:  $time_start
Features analyzed at: $time_fe
Analysis finished at: $time_end
Annotations in file $fid was saved in $fid.cds
Sequence in file $fid was saved in FASTA format in $fid.fa
";

sub extanno
{
	$intron = "ll";
	$a = $anno[0];
if ($a =~ /\s{3}(\S+)\s{3}\s+/ ) {
	$type = $1;
	if ($a =~ /complement/){ $compl = "-" } else { $compl = "+"}
	# if ($a =~ /join/){ $intron = "join" } else { $intron = "oo"}
	@a = split ",",$a;
	for $i (0..@a-1) {
		$a[$i] =~ /(\d+)\.\.\D?(\d+)/; #extract the positions of the form:  190..255
		$start[$i] = sprintf ("%8d", $1);
		$endp[$i]  = sprintf ("%8d", $2);
	}
}
else { die "Wrong \$a: $a; did not set the type of anntation" ; }

$translation = "";
$product_cont = "";
$protein_seq = "";
$gene = "";
$locus = "";
$product = "";

while ( @anno ) {
	$a = shift @anno;
	if ($a =~ /\/gene=/){
		$a =~ /\"(\w+)\"/;
		$gene = $1;
	}

	if ($a =~ /\/locus_tag=/){
		$a =~ /\"(\S+)\"/;
		$locus = $1;
	}

	if ($product_cont) {
		$a =~ /([^\".]+)/;
		chomp $product ;
		$product = $product.$1;
		$product =~ s/\s+/ /g;
		if ($a =~ /\"/) { $product_cont = "" }
	}

	if ($a =~ /\/product=/){
		$a =~ /\"([^\".]+)/;
		$product = $1;
		$product_cont = "true";
		if ($a =~ /\w\"/) { $product_cont = "" }
	}

	if ($a =~ /\/pseudo/){
		$product = "Pseudogene";
		$product_cont = "true";
		if ($a =~ /\w\"/) { $product_cont = "" }
	}

	if ($translation) {
		$a =~ /(\w+)/;
		$protein_seq = $protein_seq.$1;
		if ($a =~ /\"/) { $translation = "" }
	}

	if ($a =~ /\/translation=/){
		$a =~ /\"(\w+)/;
		$protein_seq = $1;
	$translation = "true"; }
	if ($a =~ /\w\"/) { $translation = "" }

} # end foreach $a ( @anno )

my @prot = split "", $protein_seq;
$protein_seq = ",". join (".,", @prot ) . ".***" ;
if ($compl eq "-" ){
	$protein_seq = reverse $protein_seq;
}

for $i (0..@a-1) {
	if ($type =~ /rna/ig) {$protein_seq = "r" x ($endp[$i]-$start[$i]+1) }
	$exon = "exon".($compl? @a-$i : $i+1);
	$protein_length = $endp[$i] - $start[$i] +1;
	$protein_seq_exon = substr ($protein_seq, 0, $protein_length, "");
	$anno_out = "$start[$i]\t$endp[$i]\t$compl\t$type\t$gene\t$locus $exon\t$product\t$protein_seq_exon\t\n";
	print CDS "$anno_out";  # and print to file
}

@anno = @a = @start = @endp = ();

return ();
}  #sub extanno


sub tid
{
	use Time::localtime;
	$tm = localtime;
	($Hr, $Min, $Sec) = ($tm -> hour, $tm -> min, $tm -> sec);
	$tid = sprintf("%02d:%02d:%02d",$Hr, $Min, $Sec);
	return()
} #end sub tid