#!/usr/bin/perl

# This script reformat reads in SLXA Export format to FASTQ quality base 33

#    
#    Copyright 2010, 2011 Ole Skovgaard.
#    The scripts are distributed under the terms of the GNU General Public License
#    
#    
#    This file is part of R2R - From Read to Results.
#
#    R2R is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, version 3 of the License.
#
#    R2R is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with R2R.  If not, see <http://www.gnu.org/licenses/>.
#
#    Version: Release_0.950, November, 2011
#

print "Use <*export.*> \n";

$file_seq=$ARGV[0];

unless (open(test,"<$file_seq"))  { 
print "Input file to be copied from \n\n" ;
$file_seq=<stdin>;
chomp $file_seq;
}
close test;


tid();
print "Start:$tid \nSequence file: $file_seq \n";
open(f_seq,"<$file_seq") or die "No Sequence file: $file_seq";
open out,">$file_seq.fastq";

binmode out;   # Ensures that System independent Unix "LF" (0A) is used for New Line.
$CR = chr(10);

	$go_on = "true";
	$r_no{total} = 0;
	
	while ($go_on) {
		@read  ="";		
		$seqpre = <f_seq> or $go_on = "";
		@seqpre  = split "\t",$seqpre;
		$rd = sprintf ("%08d", $r_no{total});
		$read[0] = "\@ExpRd".$rd.$CR;
		$read[1] = $seqpre[8].$CR;
		$read[2] = "+".$CR;
		@qual = split "", $seqpre[9] or $go_on = "";
		for $i (0..@qual-1) {
			$qi = ord $qual[$i];
			unless ( 58 < $qi && $qi < 105) { 
				die "Qual \$qi = $qi is out of range: \n0 $read[0];\n1 $read[1];\n2 $read[2];\n3 $read[3];\n"; # legal range for SLXA qual is 59 to 104.
			}			
			$qual = 10 * log(1-10**(-0.5) + 10 ** ( ($qi - 64) / 10.0 ) ) / log(10);
			$qual[$i] = chr( $qual + 33 );
		}
		$read[3] = join ("", @qual).$CR;
		if ($go_on) { 
			print out @read; 
			$r_no{total}++
		}
	}

tid();

print "
**************************
End: $tid
Number of reads: $r_no{total}
**************************

";


sub tid
{
	use Time::localtime;
	$tm = localtime;
	($Hr, $Min, $Sec) = ($tm -> hour, $tm -> min, $tm -> sec);
	$tid = sprintf("%02d:%02d:%02d",$Hr, $Min, $Sec);
	return()
} #end sub tid

sub Date
{
	use Time::localtime;
	$tm = localtime;
	($Day, $Month, $Year) = ($tm -> mday, $tm -> mon, $tm -> year);
	$date = sprintf("%02d-%02d-%02d",$Day, $Month+1, $Year+1900);
	return()
} #end sub Date

