#!/usr/local/bin/perl -w
# This script is specialized from eutils_example.pl found at:
# http://eutils.ncbi.nlm.nih.gov/entrez/query/static/eutils_help.html#DemonstrationPrograms
# to fetch sequences in GenBank format; here ref nucleotide sequences are fetched and their protein extracted for "eval_nucl_xxxx.pl"


# ---------------------------------------------------------------------------
# Define library for the 'get' function used in the next section.
# $utils contains route for the utilities.
# $db, $query, and $report may be supplied by the user when prompted;
# if not answered, default values, will be assigned as shown below.


#    
#    Modified from eutils_example.pl 2010 by Ole Skovgaard.
#    
#    
#    This file is part of R2R - From Read to Results.
#
#
#    Version: Release_0.910, September 20, 2010
#

print "
   This script identifies GI numbers within a CDS block in a standard 
   GenBank formatted file. Then a query is send to NCBI and hopefully
   a protein sequence is received. This sequence is added as a 
   /translation=\"xxxx section in a new file. This translation is 
   necessary for that the AnnotationExtraction_Genbank_0x.pl script 
   can extract protein sequences in the annotations.
   
   This script should only be used when the GenBank file does not
   contain the translations allready.
   ";

use LWP::Simple;

#Declarations:

my $l_in = "";
my @l_in = ();

my $list_in=$ARGV[0];
unless (open(l_in,"<$list_in"))  {
	print "Text GenBank file with Protein GI no's to be fetched \n\n" ;
	$list_in=<stdin>;
	chomp $list_in;
}
close l_in;
$list_out = $list_in."_CDS";

# open(tst1,">Test1.txt");
# open(tst2,">Test2.txt");
# open(tst3,">Test3.txt");

open(l_in,"<$list_in");
open(l_out,">$list_out");
open(f_report,">fetch_report.txt");

binmode out;   # Ensures that System independent Unix "LF" (0A) is used for New Line.

Date();
tid();
$time_start = $tid;

print "Start: $tid\n\n";
print f_report "Program: $0;
Time / Date: $tid / $date;
";

print "Translations of CDS's in $list_in will be fetched, \nand added to $list_out\n\n" ;
print f_report "\nSequences in $list_in was fetched, \n" ;
print f_report "\nResult is found in $list_out.\n\n" ;

while (<l_in>) {
	$a = $_;
	print l_out "$a";
	if ($a =~ /   \w+   /){ $cds = "" }
	if ($a =~ /   CDS   /){ $cds = "true" }
	if ($cds) {
		if ($a =~ /db_xref=\"GI:(\d+)/) {
			print "GI: $1\n";
			$translation = GI_fetch($1);
			@translation = split "\n",$translation;
			# print tst1 "GI: $1\n$translation\n";
			print l_out "                     /Fethch_GB-GI=\"$translation[0]\"\n";
			print l_out "                     /translation=\"$translation[1]\n";
			for $i (2..@translation-2){
				print l_out "                     $translation[$i]\n";
			}
			print l_out "                     $translation[@translation-1]\"\n";
		}
	}
}


tid();
print "
Finished: $tid\n";

sub GI_fetch
# This sub is specialized from eutils_example.pl found at:
# http://eutils.ncbi.nlm.nih.gov/entrez/query/static/eutils_help.html#DemonstrationPrograms

{
	$query = $_[0];

	my $utils = "http://www.ncbi.nlm.nih.gov/entrez/eutils";
	my $db     =  "protein";
	my $report =  "fasta";
	my $esearch = "$utils/esearch.fcgi?" . "db=$db&retmax=1&usehistory=y&term=";
	my $esearch_result = get($esearch . $query);
	$esearch_result =~ m|<Count>(\d+)</Count>.*<QueryKey>(\d+)</QueryKey>.*<WebEnv>(\S+)</WebEnv>|s;

	my $Count    = $1;
	my $QueryKey = $2;
	my $WebEnv   = $3;

	# print "Count = $Count; QueryKey = $QueryKey; WebEnv = $WebEnv\n";

	# ---------------------------------------------------------------------------
	# this area defines a loop which will display $retmax citation results from
	# Efetch each time the the Enter Key is pressed, after a prompt.

	my $retstart=0;
	my $retmax=1;

	my $efetch = "$utils/efetch.fcgi?" .
	"rettype=$report&retmode=text&retstart=$retstart&retmax=$retmax&" .
	"db=$db&query_key=$QueryKey&WebEnv=$WebEnv";

	my $efetch_result = get($efetch);

	return($efetch_result)
} # End GI_fetch



sub tid  # The Danish word "tid" is used instead of "time", to confuse UltraEdit
{
	use Time::localtime;
	$tm = localtime;
	($Hr, $Min, $Sec) = ($tm -> hour, $tm -> min, $tm -> sec);
	$tid = sprintf("%02d:%02d:%02d",$Hr, $Min, $Sec);
	return()
} #end sub tid

sub Date
{
	use Time::localtime;
	$tm = localtime;
	($Day, $Month, $Year) = ($tm -> mday, $tm -> mon, $tm -> year);
	$date = sprintf("%02d-%02d-%02d",$Day, $Month+1, $Year+1900);
	return()
} #end sub Date
