#!/usr/bin/perl

#    
#    Copyright 2010, 2011 Ole Skovgaard.
#    The scripts are distributed under the terms of the GNU General Public License
#    
#    
#    This file is part of R2R - From Read to Results.
#
#    R2R is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, version 3 of the License.
#
#    R2R is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with R2R.  If not, see <http://www.gnu.org/licenses/>.
#
#    Version: Release_0.950, November, 2011
#

open R2R_ini,"<R2R_read.ini"or die "The \"R2R_read.ini\" ini-file must be present in active directory";
@R2R = <R2R_ini>;
close R2R_ini;

for $i ( 0 .. @R2R-1 ) {
	@OS = split "\t",$R2R[$i];
	$OS[0] =~ /(\$\w+)\s*=\s*([-\w\.\:\ \\\/]+)/;	

	# Files:
	if ( $1 eq "\$reference_in" )    { $reference_in = $2 }
	if ( $1 eq "\$fid_core" )        { $fid_core = $2 }

	# Variables:
	$var = $2;
	$var =~ s/ //g; # spaces are not allowed in variables
	
	if ( $1 eq "\$qbase" )    	     { $qbase = $var }
	if ( $1 eq "\$ra_qlength" )      { $ra_qlength = $var }
	if ( $1 eq "\$ra_qlimit" )       { $ra_qlimit = $var }
	if ( $1 eq "\$mut_qlimit" )      { $mut_qlimit = $var }
	if ( $1 eq "\$mut_max" )         { $mut_max = $var }
	if ( $1 eq "\$init_treshold" )   { $init_treshold = $var }
	if ( $1 eq "\$unik_treshold" )   { $unik_treshold = $var }	

	############################     The variables below each determine whether
	###     Flow Control     ###     a particular section of this script will
	############################     be executed. Set to either "" or to "y"
	
	if ( $1 eq "\$PE_reads" )        { $PE_reads = $var }
}

$SR_reads = ($PE_reads ? "": "true");

$read_in               = $fid_core.".sort_reads";     
$read_out_srt          = $fid_core."_rd_srt.bex";
$morph_out             = $fid_core.".morph_out.temp";
$morph_tmp             = $fid_core.".mo.temp";
$morph_srt             = $fid_core.".mo_srt.temp";
$morph_out_srt         = $fid_core."_mo_srt.bex"; 
$morph_report          = $fid_core.".Morph_report.txt";
$tmpmo                 = $fid_core.".mut_out_bex.temp";
$mut_discarded         = $fid_core.".mut_discarded";  
$reads_mapped_no_usrt  = $fid_core.".reads_mapped_no_usrt.temp"; 
$reads_mapped_no_sort  = $fid_core.".reads_mapped_no_sort.temp"; 

check_input();

$all_reads = $fid_core.'.readno_srt';
open (all_reads,"<$all_reads") or print "Warning: no $all_reads file found; Orphan's will not be saved\n";
$dist_no_max = 10000; # Number of distances to collect for calulating average distance and variation between pairs/mates.

binmode out;   # Ensures that System independent Unix "LF" (0A) is used for New Line.
$screen = select (); # Saves the value for default output (to the screen), used after flushing of output files.

tid();  Date ();  # The Danish tid is used instead of time, to confuse UltraEdit
$time_start = $tid;

if ( $^O =~ /Win/) { 	$platform = "DOS" } else  { $platform = "Linux/Unix" } # System?
print "System:  $^O - $platform \n";

open (morph_report,">$morph_report");  
open (error,">$fid_core.errors.txt");  

 #  open (tst1,">test1.txt");  
 #  open (tst2,">test2.txt");  
 #  open (tst3,">test3.txt");  
 #  open (tst4,">test4.txt");  
 #  select (tst1);    $| = 1;     # Don't buffer print
 #  select (tst2);    $| = 1;     # Don't buffer print   
 #  select (tst3);    $| = 1;     # Don't buffer print   
 #	select (tst4);    $| = 1;     # Don't buffer print   

select ($screen);

print "$tid \tReads of $read_in will be analyzed with \n\t\t$reference_in \n\t\tas reference sequence\n\n" ;
print "Results will be in:\n   /[chromosome]/$read_out_srt and \n   /[chromosome]/$morph_out_srt \n\n";

###    Read Analysis   ###
tid();  # The Danish tid is used instead of time, to confuse UltraEdit
print "   $tid     Extract matching reads\n";
open reference_in,"<$reference_in";
open read_in,"<$read_in";	
open reads_mapped_no_usrt,">$reads_mapped_no_usrt";
open morph_out,">$morph_out";

#prolog
$ref_in[0] = <reference_in>;
chomp($ref_in[0]);
$count_ref++;
$refi = shift (@ref_in );
@refi = split ("\t", $refi);
$refi[0] =~ s/\s//g;
$rf_length = length $refi[0];
$rd_in[0] = <read_in>;
$count_rd++;
$rdi = shift (@rd_in);
@rdi = split ("\t", $rdi );

while ( @refi && @rdi ) {     ### Separates matching reads from morphs, assigns coordinate and chromosome to matching reads
	unless (@ref_in) {
		read_ref (@ref_in)
	}
	unless (@rd_in) {
		read_rd (@rd_in);
	}
	$refi[0] =~ s/\s//g;
	$rdi[0] =~ s/\s//g;
	$rd_length = length $rdi[0]; #! was length $refi[0];
	if ($rd_length > $rd_length_max) { $rd_length_max = $rd_length }
	$refi_x = substr ($refi[0], 0, $rd_length) or die "length of read exceeds length of reference index\n read:      $rdi[0]\n reference $refi[0]";
	
	$tst = ($rdi[0] cmp $refi_x);  ###
	##################################
	### compares read with reference sequence

	if ($tst == -1) {   # read is less than reference, find out whether to discard read or to send it to morph test; then get next read
		# Quality sort
		# This Q control checks after matching reads have been mapped - before MutInDel reads are mapped.
		$qual_seq = substr ($rdi[1], 0, $rd_length)  ;
		@qual = split("",$qual_seq);
		$qual = 0;
		for $i (0 .. $ra_qlength-1) {
			$qual = $qual + ord($qual[$i]);
		}
		$qual = ($qual/$ra_qlength) - $qbase;

		if ($qual >= $ra_qlimit) { 																 # Filter away criteria
			if ($rdi[0] =~ /n/ ) {  # Filter away criteria
				# $rd_out = read_lq_out;  # discard and count
				$low_qual_morphread_no++;
			}
			else {
				print morph_out "$rdi";
				$pass_morphread_no++;
			}
		}
		else {
			# $rd_out = read_lq_out;  # discard and count
			$low_qual_morphread_no++;
		}
		$rdi = shift (@rd_in);
		@rdi = split ("\t", $rdi );
		$morph_read++;  #statistics
	}
	if ($tst == 1) {   # reference is less than read, get next reference
		$refi = shift (@ref_in );
		@refi = split ("\t", $refi);
	}
	if ($tst == 0) {	# reference and read match, save match read and get next read
		my @refc = @refi;
		while (@refc > 2) {		# Maps to all occurrences of match, later only relevant match are saved
			$pos = (($refc[3] eq"+")? $refc[2]: ($refc[3] eq"-")? $refc[2]+ $rf_length - $rd_length: 0);
			$pos = sprintf("%09d",$pos);
			$readout = join ("\t", $rdi[2],$pos,$refc[1],$refc[3],$rdi[0],$rdi[1],$rdi[3],"MM0:"," ","\n" );
			print reads_mapped_no_usrt "$readout" or die "No $$reads_mapped_no_usrt file\n";
			splice (@refc,1,3);
			# counter?
		}
		$rdi = shift (@rd_in);
		@rdi = split ("\t", $rdi );
		$match_read++;  #statistics
	}

	if ($match_read =~ /000000$/) {
		if ($match_read > $match_read_prev) {
			$match_read_prev = $match_read;
			tid(); 
			my $p1 = sprintf("%12s", sub_print_No ($morph_read));
			my $p2 = sprintf("%12s", sub_print_No ($match_read));
			my $p3 = sprintf("%12s", sub_print_No ($pass_morphread_no));
			my $p4 = sprintf("%12s", sub_print_No ($low_qual_morphread_no));
			print "   $tid \t$p1 morphs and $p2 matching reads found\n";
			if($low_qual_morphread_no) { print "             $p4 low quality morps \n" }
		}
	}
} # end while ( @ref_in && @rd_in )
tid(); 
$time_match = $tid;   #Matching reads extrd:
my $p1 = sprintf("%12s", sub_print_No ($morph_read));
my $p2 = sprintf("%12s", sub_print_No ($match_read));
my $p3 = sprintf("%12s", sub_print_No ($pass_morphread_no));
my $p4 = sprintf("%12s", sub_print_No ($low_qual_morphread_no));
print "   $tid \t$p1 morphs and $p2 matching reads found\n";
if($low_qual_morphread_no) { print "             $p4 low quality morps \n" }
# Flush and close _out files;
select (morph_out);  $| = 1;  $| = 0;
select ($screen);
close morph_out;

print "   $tid     Matching reads extracted\n";

# calculate the reverse-complement of all morph reads; add them and sort them all.
# Single and many double Mutations will be found. InDel's will be asked if they are not deep in either end.

print "\t\tAdding the complement seqs to Morphs\n";
open morph_in,"<$morph_out";   # Has been flushed
open morph_temp,">$morph_out.temp";

while (<morph_in>) {
	$mo = $_;
	$mo =~ s/ //g;
	@mo_cmpl = split  ("\t",$mo);
	$mo_cmpl[0] = revComp($mo_cmpl[0]);		# sequence is reverse complement
	$mo_cmpl[1] = reverse($mo_cmpl[1]);		# quality is reversed
	$mo_cmpl[2] = $mo_cmpl[2]."c";		    # add a lc "c" to indicate complement of read
	$mo_cmpl = join ("\t",@mo_cmpl);
	print morph_temp "$mo";
	$morph_f_no++;
	print morph_temp "$mo_cmpl";
	$morph_c_no++;
}  # end while (<morph_in>)

tid();
print "   $tid\tReverse complements added to morphs\n";
select (morph_temp);  $| = 1;  $| = 0;   # Flush
select ($screen);
close morph_temp;

$sort_in_file  = $morph_out.".temp";
$sort_out_file = $morph_srt;

tid();
print "   $tid     Sorting $sort_in_file to \n\t\t$sort_out_file\n"; 

if ($platform eq "DOS" ) {
	system ("sort $sort_in_file /O $sort_out_file /L C");
}
else {	
	system ("sort $sort_in_file > $sort_out_file");
}

tid();
$time_morph_srt = $tid;   #Revers compl. morphs:
print "   $tid\tSorting finished\n";


#  Morph analysis
######################
tid();
print "   $tid \t2 x $morph_c_no Morphs will be analyzed with init_treshold = $init_treshold \n\t\tand unik_treshold = $unik_treshold \n";

# Prolog to morph analysis
open (mut_out,">$tmpmo")       or die "No file(mo): $$tmpmo;" ;  #
open (mut_discarded,">$mut_discarded")    or die "No file(md): $$mut_discarded;" ;  #
close reference_in;  # Start from 1'st line again
open(reference_in,"<$reference_in") or die "No reference_in file: $reference_in;" ;   # Needs no Flush

# open morph_tmp,">$morph_tmp" ;
open morph_srt,"<$morph_srt" or die "No morph_srt file: $morph_srt;" ;  

$morph_in_keep_it =
$ref_in_keep_it =
$seq_lead =
$ref_in_prev =
$morph_in_prev = "a" x $init_treshold;

while ( !$eof_morph || !$eof_mref ) {
	@morph_in = @ref_in =();
	if ($seq_lead eq $ref_in_prev)     { read_ref_morphs () }	
	if ($seq_lead eq $morph_in_prev)   { read_morph ()  }

	foreach $morph_in(@morph_in) {
		# Mutation? All reads with =< $mut_max SNP's are saved, No InDel's
		# this is a fishing strategy, if a SNP is not found here it will be found on the complementary strand
		# multible mutants may or may not be found
		# count number of differences between $morphi[2] and $refi[0]);

		@mut_pos = ();
		@morphi = split "\t", $morph_in; 
		$rd_length = length $morphi[0]; 
		@seq_mp = split ('', $morphi[0]);   
		$mo_depth_max = 0;  # For indel analysis

		foreach $refi(@ref_in) {
			@mut_pos = ();
			my @refc = @refi = split "\t", $refi;
			@seq_rf = split ('', $refi[0]);   
			for $i (0 .. $rd_length-1) {
				if ($seq_mp[$i] ne $seq_rf[$i]) {push @mut_pos, $i; }
			}
			$mut_no = @mut_pos;
			
			if ($mut_no <= $mut_max) {
				my ($x,$y,$z) = "";
				my $qual_pass = "";
				foreach $mut_pos(@mut_pos) {
					$x = $x."~".$mut_pos ;
					$z = substr ($morphi[1], $mut_pos, 1);
					$mut_qual = ord($z) - $qbase;
					if ($mut_qual >= $mut_qlimit) {$qual_pass = "true"}
					$y = $y."~".$z;   #Quality of mutation position
					$y =~ s/^~//;  # Removes initial "~"
				}
				# save read
				if ($qual_pass) {		
					while (@refc > 2) {
						$pos = (($refc[3] eq"+")? $refc[2]: ($refc[3] eq"-")? $refc[2]+ $rf_length - $rd_length: 0);
						$pos = sprintf("%09d",$pos);

						@readout = ( $morphi[2],$pos,$refi[1],$refi[3],$morphi[0],$morphi[1],$morphi[3],"MM".$mut_no.":".$x,$y,"\n" );
						if ($morphi[2] =~ /c$/) {@readout = rev_compl_read(@readout) }
						$readout = join "\t",@readout;						
						print reads_mapped_no_usrt "$readout" or die "No $$reads_mapped_no_usrt file\n";
						splice (@refc,1,3);
						# counter?
					}
				}
			}

			# indel analysis
			# The deepness is measured for all relevant ref-reads;
			else {
				$count_orphan_test++;
				$mo_depth = 0;
				$ref_init = substr ($refi, 0, $unik_treshold);
				$mo_init = substr ($morph_in, 0, $unik_treshold);

				# Get the "deepness" of match
				if ($mo_init eq $ref_init) {  # Or I don't care,
					my ($xxa, $xxb, $x) = "";
					$mo_depth = $unik_treshold -2; 	 # 1 for starting with 0 and 1 for $mo_depth++ being first
					while ( $xxa eq $xxb && $mo_depth < $rd_length)  {		# Works, but could be more efficient
						$mo_depth++ ;
						$xxa = substr($morph_in, $mo_depth, 1) ;
						$xxb = substr($refi, $mo_depth, 1) ;
					}
					if ( $mo_depth > $mo_depth_max ) {$mo_depth_max = $mo_depth }
					if ($refi[3] eq "+") {$x = "MM8:~"}
					if ($refi[3] eq "-") {$x = "MM9:~"}
				
					while (@refc > 2) {
						$pos = (($refc[3] eq"+")? $refc[2]: ($refc[3] eq"-")? $refc[2]+ $rf_length - $rd_length: 0);
						$pos = sprintf("%09d",$pos);
						$temp_out = join ("\t", $morphi[2],$pos,$refi[1],$refi[3],$morphi[0],$morphi[1],$morphi[3],$x.$mo_depth,$mo_depth,"\n");
						push ( @temp_out, $temp_out ) ;
						splice (@refc,1,3);
						# counter?
					}			
				}   # End if ($mo_init eq $ref_init) {  # Or I don't care,

				foreach $temp_out (@temp_out) {
					@temp_indel = split ("\t", $temp_out);
					if ( $temp_indel[8] >= $mo_depth_max) {							
						if ($temp_indel[0] =~ /c$/) {@temp_indel = rev_compl_read(@temp_indel) }
						$temp_out = join "\t",@temp_indel;							
						print reads_mapped_no_usrt "$temp_out" or die "No $$reads_mapped_no_usrt file\n";
						$count_indel_depth[$mo_depth_max]++;
						$count_indel_out++;
					}
				}   # End foreach $temp_out (@temp_out)
				@temp_out = ();  
			} # End else
			#indel analysis
		} #	End  foreach $refi(@ref_in)
	} #	End foreach $morph_in(@morph_in) {		

	if ( $cma > 1000000 ) { 
		tid(); 
		$x = sprintf  ("%14s", sub_print_No ($count_morps_anal)) ;
		$y = sub_print_No (2*$morph_c_no);
		print "   $tid \t$x of $y Morphs analyzed\n"; 
		$cma = 1;
	}
	$seq_lead = increase_seq($seq_lead);
} # End 	while ( !$eof_morph || !$eof_mref ) 

tid(); 
$x = sprintf  ("%14s", sub_print_No ($count_morps_anal)) ;
$y = sub_print_No (2*$morph_c_no);
print "   $tid \t$x of $y Morphs analyzed\n"; 

select reads_mapped_no_usrt;  $| = 1;  $| = 0;   # Flush 
close  reads_mapped_no_usrt;
select ($screen);




#*********
#Sort after read number and decide on the best matches
#*********

tid();  # The Danish tid is used instead of time, to confuse UltraEdit
$sort_in = $reads_mapped_no_usrt;
$sort_out = $reads_mapped_no_sort;
@best_store = ();
print "   $tid     Sorting mapped reads in: \n\t\t$sort_in to \n\t\t$sort_out \n";
if ($platform eq "DOS" ) {
	system ("sort $sort_in /O $sort_out /L C");
}
else {
	system ("sort $sort_in > $sort_out");
}
tid();  # The Danish tid is used instead of time, to confuse UltraEdit
print "   $tid     Sorting finished\n";	

# print un-mapped reads in FASTQ format
$orphans = $fid_core.'.orphans.fq';
open (sort_out,"<$sort_out");
open (orphans,">$orphans");	
@sort_out = split "\t", <sort_out>;
$sort_out_prev = $sort_out[0];

tid();  # The Danish tid is used instead of time, to confuse UltraEdit
print "   $tid     Saving Orphans begin\n";	

$read_all = $read_orphan = $read_matched = 0;
while (<all_reads>) {
	my $all_read = $_;
	my @all_read = split "\t", $all_read;
	$read_all++;
	if ($all_read[2] lt $sort_out[0]) {
		my $print_orphan = join ("\n","\@$all_read[2]",$all_read[0],"+",$all_read[1]);
		print orphans "$print_orphan\n";
		$read_orphan++;
	}
	else {
		while ($sort_out[0] eq $sort_out_prev && $sort_out_prev) {
			@sort_out = split "\t", <sort_out>;
			$sort_out[0] =~ s/c$//; #removes a trailing c (if complementary to read)
		}
		$sort_out_prev = $sort_out[0];
		$read_matched++;	# This is the no of mapped reads, some reads mapped more than once
	}
}

tid();  # The Danish tid is used instead of time, to confuse UltraEdit
print "   $tid     Saving Orphans finished\n";	
	
# Filter match's and put them in chr. spec. files
# Extra for PE reads: pair the reads
close sort_out;
open (sort_out,"<$sort_out");
@sort_out = split "\t",<sort_out>;		
$sort_out_prev = $sort_out[0];
my @select = ();
		
if ($SR_reads) {
	tid();  # The Danish tid is used instead of time, to confuse UltraEdit
	print "   $tid     Selecting Single Reads\n";
 	@{$select[0]} = @sort_out;  
	while (<sort_out>) {
		@sort_out = split "\t",$_ ;			
		if ($sort_out[0] eq $sort_out_prev) {
			push @select, [@sort_out];
		}
		else {				
			select_read_SR (@select);
			@select = "";   # Essential
			$sort_out_prev = $sort_out[0];				
			@{$select[0]} = @sort_out; # Avoids funny assignments # check http://perldoc.perl.org/perllol.html "Access and Printing" for this construction				
		}			
	}  # End while
	tid();  # The Danish tid is used instead of time, to confuse UltraEdit
	print "   $tid     Finished selection of Single Reads\n";
}  # End if ($SR_reads)
		
if ($PE_reads) {
	tid();  # The Danish tid is used instead of time, to confuse UltraEdit
	print "   $tid     Selecting PE reads\n";
	%ss_call = ();
 	push @select, [@sort_out]; 
	while (<sort_out>) {
		@sort_out = split "\t",$_ ;			
		$sort_out_this = $sort_out[0];
		chop $sort_out_this;	# removes trailing "a" or "b"
		if ($sort_out_this eq $sort_out_prev) {
			push @select, [@sort_out];
		}
		else {				
			$rn++;
			select_read_PE (@select);
			@select = (); 
			$sort_out_prev = $sort_out[0];				
			chop $sort_out_prev;	# removes trailing "a" or "b"
			push @select, [@sort_out]; # Avoids funny assignments # check http://perldoc.perl.org/perllol.html "Access and Printing" for this construction				
		}			
	}  # End while
	
	tid();  # The Danish tid is used instead of time, to confuse UltraEdit
	print "   $tid     Finished selection of PE reads\n";
}  # End if ($PE_reads)

close sort_out;

# Sort the matching reads after position
print  "   $tid     Sort the matching reads after chromosome and position.\n";	
foreach $chr (@chr ) {
	$opn = "ro_".$chr;
	$sort_in = $fid_core."_".$chr.".rd_out.temp";
	$sort_out = $fid_core."_rd_srt.bex";
	select ($opn);  $| = 1;  $| = 0;   # Flush
	select ($screen);
	close $opn;
	tid();
	if ($platform eq "DOS" ) {
		$sort_out = $chr.".dir\\".$sort_out; 
		print "   $tid     Sorting:\n \t\t$sort_in to \n\t\t$sort_out \n\t\tfor chromosome $chr\n"; 
		system ("sort $sort_in /O $sort_out /L C");
	}
	else {
		$sort_out = $chr.".dir/".$sort_out;
		print "   $tid     Sorting:\n \t\t$sort_in to \n\t\t$sort_out \n\t\tfor chromosome $chr\n"; 
		system ("sort $sort_in > $sort_out");
	}
}	
tid();
$time_match_srt = $tid;   #Match Read sort:
print "   $tid     Sorted the matching reads \n\t\tAdding the complement seqs to Morphs\n";

# Sort the mutation & orphan/indel reads after position
tid();
print "   $tid     Distributing morph reads for sorting\n";

foreach $chr (@chr ) {
	$fidsrt = "mo_".$chr;
	select ($fidsrt);  $| = 1;  $| = 0;   # Flush
	close $fidsrt;
}
select ($screen);

foreach $chr (@chr ) {  
	tid();  # The Danish tid is used instead of time, to confuse UltraEdit
	$sort_in = $fid_core."_".$chr.".mo_out.temp";
	$sort_out = $fid_core."_mo_srt.bex";  
	if ($platform eq "DOS" ) {
		$sort_out = $chr.".dir\\".$sort_out;
		print "   $tid     Sorting morph-sequences in \n\t\t$sort_in to \n\t\t$sort_out \n";
		system ("sort $sort_in /O $sort_out /L C");
	}
	else {
		$sort_out = $chr.".dir/".$sort_out;
		print "   $tid     Sorting morph-sequences in \n\t\t$sort_in to \n\t\t$sort_out \n";
		system ("sort $sort_in > $sort_out");
	}
	tid();  # The Danish tid is used instead of time, to confuse UltraEdit
	print "   $tid     Sorting finished\n";
	
}		# End foreach $chr (@chr ) 

close morph_srt;
close morph_in;
close mut_out;
close orphans;

tid();
$time_morph_ord = $tid;   #Morphs sorted and ord:
print "   $tid     Morphs are sorted and ordered\n";

		if ($platform eq "DOS" ) {
			system ("del *.temp");  ##
		}
		else {
			system ("rm *.temp");
		}

print_morph_report();

tid();
$time_del = $tid;   #Temp files deleted:
print "   $tid     Temp files deleted\nProgram finished\n";


# End of program
# Sub routines:

sub print_morph_report
{
	$pass_morphread_no      = sprintf  ("%8d", $pass_morphread_no) ;
	$low_qual_morphread_no  = sprintf  ("%8d", $low_qual_morphread_no) ;
	$tot_morphread_no       = sprintf  ("%8d", $pass_morphread_no + $low_qual_morphread_no) ;
	
	$count_ind_out[0]       = sprintf  ("%8d", $count_ind_out[0]) ;
	$count_ind_out[1]       = sprintf  ("%8d", $count_ind_out[1]) ;   
	$count_ref              = sprintf  ("%8d", $count_ref) ;      
	$count_rd               = sprintf  ("%8d", $count_rd) ;     
	$match_read             = sprintf  ("%8d", $match_read) ;      
	$morph_read             = sprintf  ("%8d", $morph_read) ;     
	$pass_morphread_no      = sprintf  ("%8d", $pass_morphread_no) ;      
	$low_qual_morphread_no  = sprintf  ("%8d", $low_qual_morphread_no) ;  
	$morph_f_no             = sprintf  ("%8d", $morph_f_no) ;           
	$morph_c_no             = sprintf  ("%8d", $morph_c_no) ;            
	$count_morps_anal       = sprintf  ("%8d", $count_morps_anal) ;      

	$count_dist = sprintf  ("%8d", $count_muts_dist + $count_indels_dist );  

	$read_not_orphan = $read_all - $read_orphan;
	$read_all               = sprintf  ("%8d", $read_all) ;
	$read_orphan            = sprintf  ("%8d", $read_orphan) ;
	$read_not_orphan        = sprintf  ("%8d", $$read_not_orphan) ;
	$read_matched           = sprintf  ("%8d", $read_matched) ; # Some reads mapped more than once, this valuer is not reported

	
	for $i (0..@MM-1) {
		$MM += $MM[$i];
		$MM[$i] = sprintf ("%8d", $MM[$i]) ;
	}
	$MM = sprintf ("%8d", $MM) ;
	for $i (1..7) {$MM_mut +=  $MM[$i]; }
	$MM_mut = sprintf ("%8d", $MM_mut) ;
	$MM_ind =  $MM[8] + $MM[9]; 
	$MM_ind = sprintf ("%8d", $MM_ind) ;   
	print morph_report "Program:   $0;\nDate:      $date;
	
	Run-time:  \t
	Analysis started at:   \t $time_start \t 
	Matching reads extrd:  \t $time_match \t 
	Revers compl. morphs:  \t $time_morph_srt \t 
	Reads compared:        \t $time_cmp \t 
	Match Read sorted:     \t $time_match_srt \t 
	Morphs sorted:         \t $time_morph_ord \t 
	Temp files deleted:    \t $time_del \t 
	Analysis finished at:  \t $tid \t 
	
	Statistics for experiment:\t\t  $fid_core
	Reference sequence:       \t\t  $count_ref\t
	(If full analysis: ~ 4x genome size) \t
	
	Reads read:               \t\t  $count_rd\t
	Reads matched perfectly:  \t\t  $match_read\t
	                          \t
	Reads morphed:            \t  $morph_read\t
	Low qual morphs:          \t  $low_qual_morphread_no\t 
	Passed qual morphs:       \t  $pass_morphread_no\t 
	
	Morph reads forward       \t  $morph_f_no\t
	Morph reads complemented  \t  $morph_c_no\t
	Morph's read for analysis \t  $count_morps_anal\t
	\t\t
	All saved reads:          \t\t  $MM\t
	(Some reads may be saved more than 
	once, thus 	\"All saved reads\" may 
	exceed \"No of reads matched\")
	  Saved Matching reads    \t\t  $MM[0]\t
	  Saved Mutation reads    \t\t  $MM_mut\t\n";
	  
	for $i (1..7) {
		if ($MM[$i]>0) { print morph_report "\t  Saved $i-Mutation reads \t  $MM[$i]\t\n" }
	}
	
	print morph_report "\t  Saved InDel reads      \t\t  $MM_ind\t
	  Fw strand:              \t  $MM[8]\t
	  Rv strand:              \t  $MM[9]\t
	                          \t
	Orphan statistics:
	  Total no of reads:        \t\t  $read_all
	  No of reads matched:      \t  $read_matched
	  No of orphan reads:       \t  $read_orphan
	
	Quality limit for mutations: at least one mutation in each read must exceed quality $mut_qlimit
	Quality distribution of all mutations kept: \n
	  \t     Quality\t    Kept   \n";
	for $i ( 0 .. @count_qual-1 ) {
		if ( $count_qual[$i] > 0 ) {
			$ii  = sprintf ("%3d", $i-$qbase);
			$cq  = sprintf ("%8d", $count_qual[$i]);
			print morph_report "	  Quality\t$ii: \t$cq\n";
		}
	}
		
	$count_orphan_test = sprintf  ("%8d", $count_orphan_test ) ;
	$count_indel_out = sprintf  ("%8d", $count_indel_out ) ;
	print morph_report "
	Morph reads tested for InDel reads:   $count_orphan_test\t
	Morph reads reported as InDel reads:  $count_indel_out\t
	(Some of these are discarded later in the analysis)
	Depth Statistics, treshold: [$unik_treshold] \n";
	
	for $i ($unik_treshold..$rd_length_max) {
		$count_indel_depth_tot = $count_indel_depth_tot + $count_indel_depth[$i];
		$count_indel_depth[$i] = sprintf  ("%6d", $count_indel_depth[$i] ) ;
		$ii = sprintf ("%3d", $i);
		if ($count_indel_depth[$i]) {print morph_report "\t  Depth $ii:\t$count_indel_depth[$i]\t\n"; }
	}
	print morph_report "\tTotal InDels:\t\t$count_indel_depth_tot\n";

	if ($PE_reads) {
		print morph_report "\n\nIterations for calculating mean and SD of distance between pairs
	(Values for mean distance and SD of mean distance should converge)
	Iteration:\t No of Distances included:\t Mean distance:\t SD of Mean distance: \t\n";
		for $j (0..10) {
			my $iter = sprintf ("%10d", $j);
			my $no_of_d = sprintf  ("%25d", $d{$j} );
			my $md = sprintf  ("%14.2f", $dist{$j} );
			my $SD_md = sprintf  ("%20.2f", $dist_var{$j} );
			print morph_report"	$iter\t $no_of_d\t $md\t $SD_md\t\n";
		}
	}
	
	print morph_report "	
	***********************

All Program Parameters:
 @R2R
	\n";

	print morph_report "\nERRORS:\n@morph_error";
}  # print_morph_report

sub check_input {
	# checking in files:
	unless ($fid_core) {print "\nError: No name given for this experiment; correct R2R_read.ini\n(The \$fid_core: $fid_core parameter) \n" ; exit}
	open(read_in,"<$read_in") or die "File $read_in with sorted reads is missing; correct \$fid_core in the R2R_read.ini\n" ;
	open reference_in,"<$reference_in" or die "No reference sequence was found for \$reference_in = $reference_in.\n Correct the R2R_read.ini file\n";
	close read_in, reference_in;		

	unless ($qbase)    {print "\nError: No \$qbase was given for this experiment; correct R2R_read.ini\n(\$qbase is the base number for quality\n recommende values are 33 for FastQ data and 64 for Illumina data) \n" ; exit}
	$qbase =~ s/\s//;
	if ($qbase =~ /\D+/)    {print "\nError: \$qbase = \'$qbase\' should contain digits only and be followed by \';\'; correct R2R_read.ini\n(\$qbase is the base number for quality\n recommende values are 33 for FastQ data and 64 for Illumina data) \n" ; exit}
	unless ($qbase==33 or $qbase==59 or $qbase==64)    {print "\nWarning: \$qbase is normally set to 33 for FastQ data and 59 or 64 for Illumina data\n" }

	unless ($ra_qlength)    {print "\nWarning: no \$ra_qlength was given for this experiment or it was set to zero; correct R2R_read.ini if this was unintende\n"}
	$ra_qlength =~ s/\s//;
	if ($ra_qlength =~ /\D+/)    {print "\nError: \$ra_qlength = \'$ra_qlength\' should contain digits only and be followed by \';\'; correct R2R_read.ini\n" ; exit}

	$ra_qlimit =~ s/\s//;
	if ($ra_qlimit =~ /\D+/)    {print "\nError: \$ra_qlimit = \'$ra_qlimit\' should contain digits only and be followed by \';\'; correct R2R_read.ini\n" ; exit}
	unless (0 <= $ra_qlimit and $ra_qlimit <= 40 )    {print "\nWarning: \$ra_qlimit is less than 0 or larger than 40\nThis makes no sense??\n" }

	$mut_qlimit =~ s/\s//;
	if ($mut_qlimit =~ /\D+/)    {print "\nError: \$mut_qlimit = \'$mut_qlimit\' should contain digits only and be followed by \';\'; correct R2R_read.ini\n" ; exit}
	unless (0 <= $mut_qlimit and $mut_qlimit <= 40 )    {print "\nWarning: \$mut_qlimit is less than 0 or larger than 40\nThis makes no sense??\n" }

	$mut_max =~ s/\s//;
	if ($mut_max =~ /\D+/)    {print "\nError: \$mut_max = \'$mut_max\' should contain digits only and be followed by \';\'; correct R2R_read.ini\n" ; exit}
	unless ( 0 <= $mut_max )    {print "\nWarning: \$mut_max is less than 0 \nThis makes no sense??\n" }
	unless ( $mut_max <= 7 )    {die   "\nWarning: \$mut_max is larger than 7\n7 is the maximal values allowed for \$mut_max \n" }

	$init_treshold =~ s/\s//;
	if ($init_treshold =~ /\D+/)    {print "\nError: \$init_treshold = \'$init_treshold\' should contain digits only and be followed by \';\'; correct R2R_read.ini\n" ; exit}
	unless (8 <= $init_treshold and $init_treshold <= 12 )    {print "\nWarning: \$init_treshold is less than 8 or larger than 12\nThis makes sense in special cases only.\n" }

	$unik_treshold =~ s/\s//;
	if ($unik_treshold =~ /\D+/)    {print "\nError: \$unik_treshold = \'$unik_treshold\' should contain digits only and be followed by \';\'; correct R2R_read.ini\n" ; exit}
	unless (10 <= $unik_treshold and $unik_treshold <= 25 )    {print "\nWarning: \$unik_treshold is less than 10 or larger than 25\nThis makes sense in special cases only.\n" }

	return ();
}  # sub check_input

sub read_ref
{
	$r = 1;
	while (<reference_in>) {
		s/[ ]+//g;   #remove any spaces; 
		chomp;
		push (@ref_in, $_);
		$count_ref++;
		if ( $r > 100000 ) { return(); }
		$r++;
	}
	$eof_ref = "true";
	return();
} # read_ref

sub read_rd
{
	$r = 1;
	while (<read_in>) {
		s/[ ]+//g;   #remove any spaces; PERL (under Win) has a high frequency of inserting false spaces.
		push (@rd_in, $_);
		$count_rd++;
		if ( $r > 100000 ) { return(); }
		$r++;
	}
	$eof_read = "true";
	return();
} # read_in


sub read_ref_morphs
{
	if ($ref_in_keep_it eq $seq_lead) {
		if ($ref_in_keep) {push (@ref_in, $ref_in_keep)};
		while (<reference_in>) {
			chomp;
			s/[ ]+//g;   #remove any spaces; PERL (under Win) has a high frequency of inserting false spaces.
			$ref_in_this = substr ($_, 0, $init_treshold);
			push (@ref_in, $_);
			$count_ref++;
			unless ( $ref_in_this eq $ref_in_prev ) {
				$ref_in_prev = $ref_in_this;
				$ref_in_keep = pop @ref_in;
				$ref_in_keep_it = substr ($ref_in_keep, 0, $init_treshold);
				return();  # The last reference read deviates from the prev within the first $init_treshold nt's.
			}
		}  # End while (<reference_in>)
	}  # End if ($ref_in_keep_it eq $seq_lead)
	$eof_mref = "true";
	return();
} # read_ref_morphs


sub read_morph
{
	if ($morph_in_keep_it eq $seq_lead) {
		if ($morph_in_keep) {push (@morph_in, $morph_in_keep)};
		while (<morph_srt>) {
			
			s/[ ]+//g;   #remove any spaces; PERL (under Win) has a high frequency of inserting false spaces.
			$morph_in_this = substr ($_, 0, $init_treshold);
			push (@morph_in, $_);
			$count_morps_anal++;
			$cma++;
			unless ( $morph_in_this eq $morph_in_prev ) {
				$morph_in_prev = $morph_in_this;
				$morph_in_keep = pop @morph_in;
				$morph_in_keep_it = substr ($morph_in_keep, 0, $init_treshold);
				return();  # The last reference read deviates from the prev within the first $init_treshold nt's.
			}
		}   # End while (<morph_srt>)
	}  # End if ($morph_in_keep_it eq $seq_lead)
	$eof_morph = "true";
	return();
} # read_morph


sub rev_compl_read
{
	my @mr = @_;
	my $t = $mr[3];
	if ($t eq "+") {$mr[3]= "-"}
	if ($t eq "-") {$mr[3]= "+"}
	$mr[0]  =~ s/c$//;  # removes the "c"
	$mr[4] = revComp($mr[4]);
	$mr[5] = reverse($mr[5]);

	my @mm = split ( '~', $mr[7]);
	# if ( $mm[0] eq "MM8:" ) { $mm[0] = "MM9:" }
	# if ( $mm[0] eq "MM9:" ) { $mm[0] = "MM8:" }
	
	$mr[7] = shift @mm; # truncates @mm
	@mm = reverse @mm;
	for $i (0..@mm-1) {
		$mm[$i] = $mr[6] - $mm[$i] - 1;
		$mr[7] = "$mr[7]~$mm[$i]";
	}
	return(@mr)
}  # End  rev_compl_read


sub increase_seq
{
	$seq_new = ($ref_in_prev lt $morph_in_prev ? $ref_in_prev : $morph_in_prev);
	if ($seq_new lt $seq_old) { $seq_new = $seq_old } # Prevents infinite loop
	if ($seq_new eq $seq_old) {
		$seq_new = force_increase_seq ($seq_old);
	}
	$seq_old = $seq_new;
	return ($seq_new);
}

sub force_increase_seq
{
	my $seq_no;
	my $seq = $_[0];
	my $seq_length = (length $seq) -1;
	$seq  =~ tr/acgt/0123/;
	my @seq = split "",$seq;
	for $i (0..$seq_length) {
		$seq_no = $seq_no + $seq[$seq_length-$i]*(4**$i);
	}
	$seq_no++;
	for $i (0..$seq_length) {
		$x = 4**($seq_length-$i);
		$seq_new[$i] = int ($seq_no/$x);
		$seq_no = $seq_no - $seq_new[$i] * $x;
	}
	$seq_new = join "", @seq_new;
	$seq_new  =~ tr/0123/acgt/;
	return ($seq_new);
}  # end sub force_increase_seq


sub new_chr
{
	my ($chr, $out, $rd_out) = @_;
	push @chr, $chr;	
	$chr_dir = $chr.".dir";
	tid();
	unless (opendir $chr_dir, $chr_dir ) { 
		mkdir $chr_dir or die "Error in creating directory for $chr;\n" ;
		print "   $tid \tDirectory created for Chromosome\n\t\t $chr: $chr_dir\n";
		}	
	$opn = "ro_".$chr;
	$opn_file = $fid_core."_".$chr.".rd_out.temp";
	open $opn,">$opn_file";	
	$opn = "mo_".$chr;
	$opn_file = $fid_core."_".$chr.".mo_out.temp";
	open $opn,">$opn_file";	
	print $out "$rd_out" or die "Couldn't find handle $out for printing chr $chr\n$rd_out\n$readout\n";
	return ()
} # END sub new_chr


sub select_read_PE
{
	my @sel_read     = @_;
	my @sel_read_a   = ();
	my @sel_read_b   = ();
	my %save_pos     = ();
	my @PE_flag      = ();
	%f_s             = ();  # Must be global
	for $i (0..@sel_read-1) {  # find the number of mappings there are of each read in a pair; remove dublicates
		my @a = @{$sel_read[$i]};		# check http://perldoc.perl.org/perllol.html "Access and Printing" for this construction
		my $first_second = substr $a[0], -1, 1;
		$f_s{$first_second}++;
		my $a = $a[0];
		my $pos = $a[1];
		$x = chop $a;
		if (!$save_pos{$pos} && $x eq "a") {
			$save_pos{$pos} = "true";
			push @sel_read_a, [@a];
		}
		elsif (!$save_pos{$pos} && $x eq "b") {
			push @sel_read_b, [@a];
			$save_pos{$pos} = "true";
		}
		elsif (!$save_pos{$pos}) {
			print error "Error if you came here! #1 (1' or 2' or both mates maps)\n\$x: $x; \n@{$sel_read[$i]}\n";
		}
	}
	$f_s{a} = @sel_read_a;
	$f_s{b} = @sel_read_b;
	
	# only one of a' or b' mate maps and only once.
	$f_s = $f_s{a} + $f_s{b};
	if ($f_s == 1) {
		my @a = @{$sel_read[0]};
		my $x = $a[0];
		$x = chop $x;
		@PE_flag = ($a[2], $a[1], $a[3], $x, 1, 0);
		$a[8] = join " ", @PE_flag;
		$a[9] = "SS996\n";
		$rn++;
		$rn{996}++;
		sel_save (@a);
		return();
	}
	
	# 1' AND 2' mate maps once each.
	if ( $f_s{a}  == 1 && $f_s{b}  == 1 ) {
		@PE_flag_a = @{$sel_read_a[0]};
		@PE_flag_b = @{$sel_read_b[0]};
		PE_flag();	# PE_flag takes @PE_flag_a & @PE_flag_b and returns $PE_flag_a & $PE_flag_b;
		$PE_flag_a[8] = $PE_flag_a;
		$PE_flag_b[8] = $PE_flag_b;
		$PE_flag_a[9] = "SS1116\n";
		$PE_flag_b[9] = "SS1117\n";
		$rn{1116}++;
		$rn{1117}++;
		sel_save (@PE_flag_a);
		sel_save (@PE_flag_b);
		return();
	}
	
	# else: either 1' or 2' or both mates maps more than once.
	
	if (!$PE_dist_var) {
		for $i (0..@sel_read-1) {   # Store these reads temporarily until $PE_dist_var has been set.
			my @x = @{$sel_read[$i]};
			push @best_store, [@x];
		}
		return ();
	}
	
	if (	$f_s > 0 ) {
		# if ($PE_dist) calculate else store
	
		@best_save = ();
		for $i (0..@sel_read_a-1) {   # find if one or more pairs have the "ideal" distance.
			@sr_a = @{$sel_read_a[$i]};
			for $j (0..@sel_read_b-1) {
				@sr_b = @{$sel_read_b[$j]};
				if ( $sr_a[2] eq $sr_b[2] ) {	   # Same chromosome
					if ( $sr_a[3] ne $sr_b[3] ) {  # Different strand
						my $dist_dev_a_b = abs( abs($sr_a[1] - $sr_b[1]) - $PE_dist );
						# find and save if one or more pairs have the "ideal" distance.
						if ($dist_dev_a_b < 4*$PE_dist_var) { # An "ideal" distance; save [first occurence]and return
							$bs = join "\t", 1, $i, $j, "\n";
							push @best_save, $bs;
						}
					}
				}
			}
		}  # End for $i (0..@sel_read_a-1)
	
		if (@best_save) {
			@best_save 	= sort  {$a <=> $b} @best_save;
			#	for $i(0..@best_save-1) {		# Save first = lowest coordinates of "ideal distance pairs"
			@b_s = split "\t", $best_save[0];
			my $aa = $b_s[1];
			my $bb = $b_s[2];
			@PE_flag_a = @{$sel_read_a[$aa]};
			@PE_flag_b = @{$sel_read_b[$bb]};
			PE_flag();	# PE_flag takes @PE_flag_a & @PE_flag_b and returns $PE_flag_a & $PE_flag_b;
			$PE_flag_a[8] = $PE_flag_a;
			$PE_flag_b[8] = $PE_flag_b;
			$PE_flag_a[9] = "SS1165\n";
			$PE_flag_b[9] = "SS1166\n"; #for test
			$rn{1165}++;
			$rn{1166}++;
			sel_save (@PE_flag_a);
			sel_save (@PE_flag_b);
			return();  # This return ensures that only the first (lowest coordinate on first chr) is saved; this is for "ideal distances" only
			#	}
		}
	
		# no pair had the "ideal" distance on opposite strands.
		# Save first occurence of "a" and "b" reads, that remained unpaired without any attempts to pair them
				
		@sel_idx = ();
		for $i (0..@sel_read_a-1) {  
			my @a = @{$sel_read_a[$i]};		# check http://perldoc.perl.org/perllol.html "Access and Printing" for this construction
			@sel_idx[$i] = substr ($a[7],2,1) ;
			if ( @sel_idx[$i] == 9 ) {@sel_idx[$i] = 8}
			@sel_idx[$i] .= $a[0];
		}
		@sel_read_a = @sel_read_a [ sort {$sel_idx[$a] cmp $sel_idx[$b] } 0..@sel_read_a-1 ]; 
	
		@sel_idx = ();
		for $i (0..@sel_read_b-1) {  
			my @a = @{$sel_read_b[$i]};		# check http://perldoc.perl.org/perllol.html "Access and Printing" for this construction
			@sel_idx[$i] = substr ($a[7],2,1) ;
			if ( @sel_idx[$i] == 9 ) {@sel_idx[$i] = 8}
			@sel_idx[$i] .= $a[0];
		}
		@sel_read_b = @sel_read_b [ sort {$sel_idx[$a] cmp $sel_idx[$b] } 0..@sel_read_b-1 ]; 
	
	
	
		@sr = @{$sel_read_a[0]};
		if ($sr[0]) {
			$PE_flag[0] = $sr[2];
			$PE_flag[1] = $sr[1];
			$PE_flag[2] = $sr[3];
			$PE_flag[3] = "a";
			$PE_flag[4] = $f_s{a};
			$PE_flag[5] = 0;
	
			$PE_flag[10] =
			$PE_flag[11] =
			$PE_flag[12] = 
			$PE_flag[13] = 
			$PE_flag[14] = "";
	
			$sr[8] = join " ", @PE_flag;
			$sr[9] = "SS1194\n";
			$rn{1194}++;
			sel_save (@sr);
		}
		
		@sr = @{$sel_read_b[0]};
		if ($sr[0]) {
			$PE_flag[0] = $sr[2];
			$PE_flag[1] = $sr[1];
			$PE_flag[2] = $sr[3];
			$PE_flag[3] = "b";
			$PE_flag[4] = $f_s{b};
			$PE_flag[5] = 0;
	
			$PE_flag[10] =
			$PE_flag[11] =
			$PE_flag[12] = 
			$PE_flag[13] = 
			$PE_flag[14] = "";
		
			$sr[8] = join " ", @PE_flag;
			$sr[9] = "SS1215\n";
			$rn{1215}++;
			sel_save (@sr);
		}		
		return(); 	# END   # no pair had the "ideal" distance. Save unpaired reads
	}
	return(); 	# This return is an emergency after all other pairs have been saved;
} # End sub select_read_PE


sub select_read_SR
{
	my @sel_read = @_;
	my $pos_min = "";

	# if only 1 mapping of this read; then this mapping will be the best:
	if (@sel_read == 1) {
		my @a = @{$sel_read[0]};
		$a[9] = "SR1219\n";
		sel_save (@a);
		return();
	}

	# if one or more reads match?	1) find reads with lowest number of mismatch; 2) find first position
	@sel_idx = ();
	for $i (0..@sel_read-1) {
		my @a = @{$sel_read[$i]};		# check http://perldoc.perl.org/perllol.html "Access and Printing" for this construction
		$sel_idx_7[$i] = substr ($a[7],2,1) ;
		if ($sel_idx_7[$i] == 9) {$sel_idx_7[$i] = 8} ;
		$sel_idx_2[$i] = $a[2];
		$sel_idx_1[$i] = $a[1];
		$sel_idx[$i] = join "\t",$sel_idx_7[$i],$sel_idx_2[$i],$sel_idx_1[$i];
	}
	@sel_idx = sort {$a cmp $b} @sel_idx;
	($sel_idx_7, $sel_idx_2, $sel_idx_1) = split "\t", $sel_idx[0];
	for $i (0..@sel_read-1) {
		my @a = @{$sel_read[$i]};		# check http://perldoc.perl.org/perllol.html "Access and Printing" for this construction
		if ( $sel_idx_1 eq $a[1] && $sel_idx_2 eq $a[2] && ( $sel_idx_7 eq substr ($a[7],2,1) || $sel_idx_7 == 8 && substr ($a[7],2,1) == 9 )  )  {
			$a[9] = "SR1250\n";
			sel_save (@a);
			return();
		}
	}
	print error "Error if you came here! #3\n@{$sel_read[0]}\n";
	return();
}  #End sub select_read_SR

	
sub sel_save
{
	my @sel_save = @_;
	my @mut_pos = ();
	my $chr = $sel_save[2];
	$sel_save[7] =~ /MM(\d+):/ ;
	$MM[$1]++;
	$sel_save[7] =~ /MM(\d)/;
	my $out = ($1 == 0? "ro_".$chr : $1 >= 0? "mo_".$chr : "error") ;	
	if ($1 > 0 && $1 < 8) {
		@mut_pos = split "~",$sel_save[7];
		for $i (1..@mut_pos-1) {
			$mut_qual = ord ( substr ($sel_save[5], $mut_pos[$i], 1) );
			$count_qual[$mut_qual]++;
		}
	}
	my $rd = shift @sel_save;
	splice (@sel_save,3,0,$rd);
	my $save = join ("\t",@sel_save);	
	print $out "$save" or new_chr ($chr, $out, $save);
	$ss_call{$sel_save[9]}++;
	return();
}  # End sub sel_save


sub calculate_PE_distance
{
	$j=0;
	$d{$j} = @distances;	
	for $i (0..$d{$j}) {
		$distances[$i] = abs $distances[$i];
		$dist{$j} += $distances[$i];
	}
	$dist{$j} = $dist{$j} / ($d{$j}); # average distance;
	for $i (0..@distances-1) {
		 $dist_var{$j} += ($distances[$i]-$dist{$j})**2  ;
	}
	$dist_var{$j} = sqrt ($dist_var{$j}/@distances-2);
	
	for $j (1..10) {	
		for $i (0..@distances-1) {
			if (abs ($distances[$i]-$dist{$j-1}) < 3*$dist_var{$j-1} ) {
				$dist{$j} += $distances[$i];
				$d{$j}++;
			}
		}		
		# print "\n1 $j; 2 $dist{$j-1}; 3 $dist{$j}; ";		
		$PE_dist = $dist{$j} = $dist{$j} / $d{$j}; # average distance
		for $i (0..@distances-1) {
			if (abs ($distances[$i]-$dist{$j}) < 3*$dist_var{$j-1} ) {		
				$dist_var{$j} += ($distances[$i]-$dist{$j})**2  ;
			}
		}
		$PE_dist_var = $dist_var{$j} = sqrt ($dist_var{$j}/$d{$j}-1);		
	}	
	return ();
}

sub PE_flag
{
	my $xa = $PE_flag_a[0];
	my $xb = $PE_flag_b[0];
	$PE_flag[0] = $PE_flag_a[2];
	$PE_flag[1] = $PE_flag_a[1];
	$PE_flag[2] = $PE_flag_a[3];
	$PE_flag[3] = chop $xa;
	$PE_flag[4] = $f_s{a};

	$PE_flag[10] = $PE_flag_b[2];
	$PE_flag[11] = $PE_flag_b[1];
	$PE_flag[12] = $PE_flag_b[3];
	$PE_flag[13] = chop $xb;
	$PE_flag[14] = $f_s{b};

	# if ( $PE_flag_a[0] eq $PE_flag_b[0] ) {	   # Same chromosome
	if ( $PE_flag_a[2] eq $PE_flag_b[2] ) {	   # Same chromosome
		if ( $PE_flag_a[3] ne $PE_flag_b[3] ) {  # Different strand
			$PE_flag[5] = $PE_flag_b[1] - $PE_flag_a[1];
			$PE_flag[15] = -$PE_flag[5] ;
			
			# Collect "Ideal distances":
			if ($dist_no < $dist_no_max && !$PE_dist ) {
				push @distances, abs $PE_flag[5];
				$dist_no++;
				if ($dist_no == $dist_no_max) {
					# calculate_PE_distance
					$j=0;
					$d{$j} = @distances;
					for $i (0..$d{$j}) {
						$distances[$i] = abs $distances[$i];
						$dist{$j} += $distances[$i];
					}
					$dist{$j} = $dist{$j} / ($d{$j}); # average distance;
					for $i (0..@distances-1) {
						$dist_var{$j} += ($distances[$i]-$dist{$j})**2  ;
					}
					$dist_var{$j} = sqrt ($dist_var{$j}/@distances-2);
	
					for $j (1..10) {
						for $i (0..@distances-1) {
							if (abs ($distances[$i]-$dist{$j-1}) < 3*$dist_var{$j-1} ) {
								$dist{$j} += $distances[$i];
								$d{$j}++;
							}
						}
						$PE_dist = $dist{$j} = $dist{$j} / $d{$j}; # average distance
						for $i (0..@distances-1) {
							if (abs ($distances[$i]-$dist{$j}) < 3*$dist_var{$j-1} ) {
								$dist_var{$j} += ($distances[$i]-$dist{$j})**2  ;
							}
						}
						$PE_dist_var = $dist_var{$j} = sqrt ($dist_var{$j}/$d{$j}-1);
					}
					best_store_empty (); # Re-calculate first $dist_no_max reads with the now known mean and SD of good Paired Read distances
				}
			}	# End calculate_PE_distance
		}
	}
	else {
		$PE_flag[5] = $PE_flag[15] = 0;
	}
	$PE_flag_a = join " ", @PE_flag;
	@PE_flag_temp = splice (@PE_flag, 0, 10);
	for $i (0..@PE_flag_temp) {$PE_flag[$i+10] = $PE_flag_temp[$i]}
	$PE_flag_b = join " ", @PE_flag;
	return ();
}	# End sub PE_flag

sub best_store_empty
{
	for $i (0..@best_store-1) {
		my $bs = @{best_store[$i]};	
		@sort_out = @$bs;		
		$sort_out_this = $sort_out[0];
		chop $sort_out_this;	# removes trailing "a" or "b"
		if ($sort_out_this eq $sort_out_prev) {
			push @select, [@sort_out];
		}
		else {				
			$rn++;
			select_read_PE (@select);
			@select = (); 
			$sort_out_prev = $sort_out[0];				
			chop $sort_out_prev;	# removes trailing "a" or "b"
			push @select, [@sort_out]; # Avoids funny assignments # check http://perldoc.perl.org/perllol.html "Access and Printing" for this construction				
		}			
	}
	@best_store = ();		
	return ();
}	# End sub best_store_empty

sub revComp
{
	my $r=reverse($_[0]);
	$r=~tr/ACGTacgtNn/TGCAtgcaNn/;
	return  $r;
} #revComp

sub sub_print_No {
	my $No = $_[0];
	my $No_l = (length $No) -1;
	my $t_3 = - int ($No_l / 3) - 1;
	my $No_pr = "";
	while ( $t_3 < 0 ) {
		$No_pr .= substr ($No, 3*$t_3,3).",";
		$t_3++;
	}
	chop $No_pr;
	return($No_pr);
} # END sub sub_print_No

sub tid
{
	use Time::localtime;
	$tm = localtime;
	($Hr, $Min, $Sec) = ($tm -> hour, $tm -> min, $tm -> sec);
	$tid = sprintf("%02d:%02d:%02d",$Hr, $Min, $Sec);
	return()
} #end sub tid

sub Date
{
	use Time::localtime;
	$tm = localtime;
	($Day, $Month, $Year) = ($tm -> mday, $tm -> mon, $tm -> year);
	$date = sprintf("%02d-%02d-%02d",$Day, $Month+1, $Year+1900);
	return()
} #end sub Date

