#!/usr/bin/perl

#    
#    Copyright 2010, 2011 Ole Skovgaard.
#    The scripts are distributed under the terms of the GNU General Public License
#    
#    
#    This file is part of R2R - From Read to Results.
#
#    R2R is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, version 3 of the License.
#
#    R2R is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with R2R.  If not, see <http://www.gnu.org/licenses/>.
#
#    Version: Release_0.950, November, 2011
#


open R2R_read,"<R2R_read.ini" or die "The \"R2R_read.ini\" ini-file must be present in active directory";
@R2R = <R2R_read>;
close R2R_read;
for $i ( 0 .. @R2R-1 ) {
	@OS = split "\t",$R2R[$i];
	$OS[0] =~ /(\$\w+)\s*=\s*([-\w\.\:\ \\\/]+)/;

	# Files:
	#Single End file:
	if ( $1 eq "\$fid_fq" )     { $fid_fq = $2 }
	if ( $1 eq "\$fid_ql" )     { $fid_ql = $2 }
	if ( $1 eq "\$fid_sp" )     { $fid_sp = $2 }
	
	#Paired End files:
	if ( $1 eq "\$fid_fq_1" )   { $fid_fq_1 = $2 }
	if ( $1 eq "\$fid_fq_2" )   { $fid_fq_2 = $2 }
	if ( $1 eq "\$fid_sp_1" )   { $fid_sp_1 = $2 }
	if ( $1 eq "\$fid_ql_1" )   { $fid_ql_1 = $2 }
	if ( $1 eq "\$fid_sp_2" )   { $fid_sp_2 = $2 }
	if ( $1 eq "\$fid_ql_2" )   { $fid_ql_2 = $2 }

	# Variables:
	if ( $1 eq "\$fid_core" )   { $fid_core = $2 }
	if ( $1 eq "\$qbase" )    	{ $qbase = $2 }
	if ( $1 eq "\$qlength" )    { $qlength = $2 }
	if ( $1 eq "\$qlimit" )     { $qlimit = $2 }
}

tid();  # The Danish tid is used instead of time, to confuse UltraEdit
$time_start = $tid;
Date();

check_input ();

$screen = select (); # Saves the value for default output (to the screen), used after flushing of output files.
binmode out;   # Ensures that System independent Unix "LF" (0A) is used for New Line.

if ( $^O =~ /Win/) { 	$platform = "DOS" } else  { $platform = "Linux/Unix" } # System?
print "System: $^O - $platform \n";

open rd_report,">$fid_core.Read_report.txt";
open readno_srt,">$fid_core.readno_srt";
open read_lq_out,">$fid_core.lq_reads.fq";

if ($fid_fq =~ /\w/) {	read_fastq() }
if ($fid_sp =~ /\w/) {	pair_seq_qual() }
if ($fid_sp_1 =~ /\w/) {	pair_seq_qual_PE() }
if ($fid_fq_1 =~ /\w/) {	read_fastq_PE() }

tid();
$time_dist = $tid;
print "Reads has been assembled and tested: $tid\n";

select readno_srt; $| = 1; $| = 0; 	# auto flush
close readno_srt;
select $screen; #after auto flush

tid();
$tid_sort1_start = $tid;
$sort_in = "$fid_core.readno_srt";
$sort_out = "$fid_core.sort_reads";
print "$tid Sorting reads in $sort_in to $sort_out \n";

if ($platform eq "DOS" ) {
	system ("sort $sort_in /O $sort_out /L C");
}
else {
	system ("sort $sort_in > $sort_out");
}

tid();
$time_srt = $tid;
print "Reads has been sorted: $tid\n";
tid();
$time_morph_ord = $tid;   #Morphs sorted and ord:
print "   $tid     Morphs are sorted and ordered\n";

tid();
$time_del = $tid;
$x1 = "Reads of $fid_fq were quality assessed and indexed.";
$x2 = "Reads of $fid_sp were merged with qualities from $fid_ql, then they were quality assessed and and indexed.";
$x3 = "Reads of $fid_sp_1 were merged with qualities from $fid_ql_1, \nReads of $fid_sp_2 was merged with qualities from $fid_ql_2, \nthen all reads were quality assessed and indexed for paired end analysis.";
$x4 = "Reads of $fid_fq_1 and $fid_fq_2, \nand all reads were quality assessed \nand indexed for paired end analysis.";

$xprint = ($fid_fq? $x1: $fid_sp?$x2: $fid_sp_1?$x3: $fid_fq_1?$x4: "");

print rd_report "Program: $0
Date: $date     Time: $tid.

$xprint

Quality was better than $qlimit on the first $qlength bases;
Parameters are set in the R2R_read.ini file

Outfiles:
Sorted reads in:             $fid_core.sort_reads
Low quality reads in:        $fid_core.lq_reads.fq

Analysis started at:         $time_start
Reads assembled & filtered:  $time_dist
Reads sorted:                $time_srt
Analysis finished at:        $tid

Reads statistics:
";

$r_no{total} = sprintf  ("%10d", $r_no{total}) ;
if ($fid_sp_1.$fid_fq_1 =~ /\w/) {   # if paired end ??
	$r_no_tot = 2*$r_no{total};
	$r_no_tot = sprintf  ("%10d", $r_no_tot ) ;
	print rd_report     "Total pairs:               $r_no{total} \nTotal reads:               $r_no_tot ";
}
else {
	print rd_report "Total reads:                     $r_no{total} ";
	}

$pq_r_no      = sprintf  ("%10d", $r_no{readno_srt} ) ;
$lq_r_no      = sprintf  ("%10d", $r_no{read_lq_out} ) ;

print rd_report "
Passed reads:              $pq_r_no
Low quality reads:         $lq_r_no

Running Parameters:
@R2R
";
# end

sub check_input {
	# checking in files:
	my $in_f_no = 0;
	if ($fid_sp =~ /\w/) {
		$in_f_no++;
		open(GA_seqpre,"<$fid_sp")   or die  "GA file: $fid_sp; with seqpre-Reads to be indexed is missing, correct R2R_read.ini\n" ;
		open(GA_qual,"<$fid_ql")  or die  "GA file: $fid_ql with qcal of Reads to be indexed, correct R2R_read.ini\n" ;
		print	"\nExperiment/Sample: $fid_core\n	Reads of $fid_sp will be merged with $fid_ql\n	qualities then they will be quality assessed and indexed.\n	Analysis started: $date $tid\n\n" ;
	}	
	if ($fid_sp_1 =~ /\w/) {
		$in_f_no++;
		open(GA_seqpre_1,"<$fid_sp_1")   or die  "GA file: $fid_sp_1  with seqpre-Reads _1 to be indexed is missing, correct R2R_read.ini\n" ;
		open(GA_qual_1,"<$fid_ql_1")     or die  "GA file: $fid_ql_1 with qual of Reads _1 to be indexed, correct R2R_read.ini\n" ;
		open(GA_seqpre_2,"<$fid_sp_2")   or die  "GA file: $fid_sp_2  with seqpre-Reads _2 to be indexed is missing, correct R2R_read.ini\n" ;
		open(GA_qual_2,"<$fid_ql_2")     or die  "GA file: $fid_ql_2 with qual of Reads _2 to be indexed, correct R2R_read.ini\n" ;
		print	"\nPaired End Experiment/Sample: $fid_core\n	Reads will be merged with quality scores\n	qualities then they will be quality assessed and indexed.\n	Analysis started: $date $tid\n\n" ;
	}
	if ($fid_fq =~ /\w/) {
		$in_f_no++;
		open(GA_fastq,"<$fid_fq")   or die  "GA file: $fid_fq  with fastq Reads to be indexed is missing, correct R2R_read.ini\n" ;
		print	"\nExperiment/Sample: $fid_core\n	Reads of $fid_fq\n	will be quality assessed and indexed.\n	Analysis started: $date $tid\n\n" ;
	}
	if ($fid_fq_1 =~ /\w/) {
		$in_f_no++;
		open(GA_fastq_1,"<$fid_fq_1")   or die  "GA file: $fid_fq_1  with fastq Reads to be indexed is missing, correct R2R_read.ini\n" ;
		open(GA_fastq_2,"<$fid_fq_2")   or die  "GA file: $fid_fq_2  with fastq Reads to be indexed is missing, correct R2R_read.ini\n" ;
		print	"\nPaired End Experiment/Sample: $fid_core\n	Reads of $fid_fq_1 and $fid_fq_2\n	will be quality assessed and indexed.\n	Analysis started: $date $tid\n\n" ;
	}

	if ( $in_f_no == 0 ) { print "\nError: No infile was given; correct R2R_read.ini\n(The \$fid_ parameters)\n" ; exit }
	if ( $in_f_no > 1 ) { print "\nError: Too many infile were given; correct R2R_read.ini\n(The \$fid_ parameters)\n" ; exit }
	
	#checking other variables:
	unless ($fid_core) {print "\nError: No name given for this experiment; correct R2R_read.ini\n(The \$fid_core parameter) \n" ; exit}
	unless ($qbase)    {print "\nError: No \$qbase was given for this experiment; correct R2R_read.ini\n(\$qbase is the base number for quality\n recommende values are 33 for FastQ data and 64 for Illumina data) \n" ; exit}
	$qbase =~ s/\s//;
	if ($qbase =~ /\D+/)    {print "\nError: \$qbase = \'$qbase\' should contain digits only and be followed by \';\'; correct R2R_read.ini\n(\$qbase is the base number for quality\n recommende values are 33 for FastQ data and 64 for Illumina data) \n" ; exit}
	unless ($qbase==33 or $qbase==64)    {print "\nWarning: \$qbase is normally set to 33 for FastQ data and 64 for Illumina data\n" }

	unless ($qlength)    {print "\nWarning: no \$qlength was given for this experiment or it was set to zero; correct R2R_read.ini if this was unintende\n"}
	$qlength =~ s/\s//;
	if ($qlength =~ /\D+/)    {print "\nError: \$qlength = \'$qlength\' should contain digits only and be followed by \';\'; correct R2R_read.ini\n" ; exit}

	$qlimit =~ s/\s//;
	if ($qlimit =~ /\D+/)    {print "\nError: \$qlimit = \'$qlimit\' should contain digits only and be followed by \';\'; correct R2R_read.ini\n" ; exit}
	unless (0 <= $qlimit and $qlimit <= 40 )    {print "\nWarning: \$qlimit is less than 0 or larger than 40\nThis makes no sense??\n" }

	return ();
}  # sub check_input


sub read_fastq # Reformat sequence and quality from FASTQ format files for Single End reads
{
	$go_on = "true";
	$r_no{total} = 0;
	while ($go_on) {
		@read = "";
		$read[2] = "SR_" . sprintf ("%08d", $r_no{total}); # SR_ - Single Read & split character
		$empty1  = <GA_fastq> ;
		$read[0] = lc <GA_fastq> ;
		$empty2  = <GA_fastq> ;
		$read[1] = <GA_fastq> ;
		$go_on = ($read[0] && $read[1]);
		unless ($go_on) {next}
		chomp ($read[0],$read[1]);
		if (substr ($read[0], -1) =~ /\W/) {chop $read[0]; chop $read[1]}  # removes hex(0D) / CR from Unix/Linux
		$read[3] = length $read[0];   # one could check that length of sequence and quality is the same.
		$read[4] = "\t\n";
		$rd_out = readno_srt;
		$read = join ("\t",@read);

		if ( $qlength > 0 ) {
			@qual = split("",$read[1]);
			$qual = 0;
			for ($i = 0; $i < $qlength; $i++ ) {
				$qual = $qual + ord($qual[$i]) - $qbase;
			}
			$qual = $qual / $qlength;
			if ($qual < $qlimit) { 																 # Filter away criteria
				$read = $empty1.$read[0]."\n".$empty2.$read[1]."\n";
				$rd_out = read_lq_out;
			}
		}	# End if $qlength > 0

		print $rd_out "$read";	
		$r_no{$rd_out}++;
		$r_no{total}++;
			
		if (substr ($r_no{total}, -6) == 0 ) {
			$rt_print = sprintf "%12s", sub_print_No ($r_no{total});
			$rs_print = sprintf "%12s", sub_print_No ($r_no{readno_srt});
			$rl_print = sprintf "%12s", sub_print_No ($r_no{read_lq_out} );
			tid();
			print "$tid$rt_print reads;$rs_print passed;$rl_print low quality\n"
		}
	}	# End while go_on
	
	$rt_print = sprintf "%12s", sub_print_No ($r_no{total});
	$rs_print = sprintf "%12s", sub_print_No ($r_no{readno_srt});
	$rl_print = sprintf "%12s", sub_print_No ($r_no{read_lq_out} );
	tid();
	print "$tid$rt_print reads;$rs_print passed;$rl_print low quality\n";
	
	read_check:
	if ($read[0] || $read[1]){
		print "
		Warning: unequal no's of seq's  and qual's
		Seq:  $read[0]?
		Qual: $read[1]?
		Read: $read[3]?
		";
	}
	return ();
}  # sub read_fastq


sub pair_seq_qual # Pairs sequence and quality from seqpre and qcal/qraw Illumina files for Single End reads
{
	$go_on = "true";
	$r_no{total} = 0;
	while ($go_on) {
		@read  ="";
		$read[2] = "SR_" . sprintf ("%08d", $r_no{total}); # SR_ - Single Read & split character
		$seqpre = lc <GA_seqpre> or goto read_check;
		@seqpre  = split "\t",$seqpre;
		$read[0] = $seqpre[1];
		$read[1] = <GA_qual> or goto read_check;
		$go_on = ($read[0] && $read[1]);
		unless ($go_on) {next}
		chomp ($read[0],$read[1]);
		if (substr ($read[0], -1) =~ /\W/) {chop $read[0]; chop $read[1]}  # removes hex(0D) / CR from Unix/Linux
		$read[3] = length $read[0];   # one could check that length of sequence and quality is the same.
		$read[4] = "\t\n";
		$rd_out = readno_srt;

		if ( $qlength > 0 ) {
			@qual = split("",$read[1]);
			$qual = 0;
			for ($i = 0; $i < $qlength; $i++ ) {
				$qual = $qual + ord($qual[$i]) - $qbase;
			}
			$qual = $qual / $qlength;
			if ($qual < $qlimit) { 																 # Filter away criteria
				$rd_out = read_lq_out;
			}
		}	# End if ( $qlength > 0 )
	
		$read = join ("\t",@read);
		print $rd_out "$read";	
		$r_no{$rd_out}++;
		$r_no{total}++;
	
	if (substr ($r_no{total}, -6) == 0 ) {
		tid();
		print "$tid  $r_no{total} reads; $r_no{readno_srt} reads passed; $r_no{read_lq_out} were of low quality\n"
	}
	}	# End while go_on
	read_check:
	if ($seqpre || $read[1]){
		print "
		Warning: unequal no's of seq's  and qual's
		Seq:  $read[0]?
		Qual: $read[1]?
		";
		return ();
	}
	return ();
}  # sub pair_seq_qual


sub pair_seq_qual_PE  # Pairs sequence and quality from seqpre and qcal/qraw Illumina files for Paired End reads
{
	$go_on = "true";
	$r_no = 0;
	$read_check = "";
	while ($go_on) {
		@read_1 = @read_2 = "";
		$seqpre_1 = lc <GA_seqpre_1> or $read_check = "true";
		$seqpre_2 = lc <GA_seqpre_2> or $read_check = "true";
		$go_on = ($seqpre_1 && $seqpre_2);
		unless ($go_on) {next}		
		@seqpre_1 = split ("\t", $seqpre_1);
		@seqpre_2 = split ("\t", $seqpre_2);
		$read_1[0] = $seqpre_1[1];
		$read_2[0] = $seqpre_2[1];
		$read_1[1] = <GA_qual_1> or $read_check = "true";
		$read_2[1] = <GA_qual_2> or $read_check = "true";
		if ($read_check) {goto read_check}
		chomp ($read_1[0],$read_1[1]);
		if (substr ($read_1[0], -1) =~ /\W/) {chop $read_1[0]; chop $read_1[1]}  # removes hex(0D) / CR from Unix/Linux
		chomp ($read_2[0],$read_2[1]);
		if (substr ($read_2[0], -1) =~ /\W/) {chop $read_2[0]; chop $read_2[1]}  # removes hex(0D) / CR from Unix/Linux

		$read_1[2] = "PR_" . sprintf ("%08d", $r_no{total}) . "a";
		$read_2[2] = "PR_" . sprintf ("%08d", $r_no{total}) . "b";
		$read_1[3] = length $read_1[0];
		$read_2[3] = length $read_2[0];
		$read_1[4] = $read_2[4] = "\t\n";

		@qual_1 = split("",$read_1[1]);
		@qual_2 = split("",$read_2[1]);
		$qual_1 = $qual_2 = 0;
		for ($i = 0; $i < $qlength; $i++ ) {
			$qual_1 = $qual_1 + ord($qual_1[$i]) - $qbase;
			$qual_2 = $qual_2 + ord($qual_2[$i]) - $qbase;
		}
		$qual_1 = $qual_1 / $qlength;
		$qual_2 = $qual_2 / $qlength;

		if ($qual_1 > $qlimit) { 																 # Filter away criteria
			if ( $read_1[0] =~ /a{25}/ || $read_1[0] =~ /n/ ) {    # Filter away criteria
				$rd_out_1 = read_lq_out;
			}
			else {
				$rd_out_1 = readno_srt;
			}
		}
		else {
			$rd_out_1 = read_lq_out;
		}
	
		if ($qual_2 > $qlimit) { 																 # Filter away criteria
			if ( $read_2[0] =~ /a{25}/ || $read_2[0] =~ /n/ ) {    # Filter away criteria
				$rd_out_2 = read_lq_out;
			}
			else {
				$rd_out_2 = readno_srt;
			}
		}
		else {
			$rd_out_2 = read_lq_out;
		}
		
		$seqread_1 = join ("	",@read_1);
		print $rd_out_1 "$seqread_1";		
		$seqread_2 = join ("	",@read_2);
		print $rd_out_2 "$seqread_2";		
		$r_no{$rd_out_1}++;
		$r_no{$rd_out_2}++;
		$r_no{total}++;
		
	if (substr ($r_no{total}, -6) == 0 ) {
			tid();
			print "$tid   $r_no{total} paired reads have been pre-assembled\n"
		}
	}	# End while go_on
	read_check:
	if ($read_1[0] || $read_1[1] || $read_2[0] || $read_2[1] ){
		print "\nWarning: unequal no's of seq's  and qual's\n  Seq_1:  @read_1?\n  Seq_2:  @read_2?\n \n
		$seqpre_1
		$seqpre_2
		";
	}
	return ();
}  # sub pair_seq_qual_PE


sub read_fastq_PE {	# Reformat sequence and quality from a pair of FASTQ format files for Paired End reads	
	$go_on = "true";
	$r_no = 0;
	$read_check = "";
	while ($go_on) {
		@read_1 = @read_2 = "";		
		
		$read_1[2] = "PR_" . sprintf ("%08d", $r_no{total}) . "a"; # PR_ - Paired Read & split character
		$empty  = <GA_fastq_1> ;
		$read_1[0] = lc <GA_fastq_1> ;
		$empty  = <GA_fastq_1> ;
		$read_1[1] = <GA_fastq_1> ;	
			
		$read_2[2] = "PR_" . sprintf ("%08d", $r_no{total}) . "b"; # PR_ - Paired Read & split character
		$empty  = <GA_fastq_2> ;
		$read_2[0] = lc <GA_fastq_2> ;
		$empty  = <GA_fastq_2> ;
		$read_2[1] = <GA_fastq_2> ;	
		
		$go_on = ($read_1[0] && $read_1[1] && $read_2[0] && $read_2[1]);
		unless ($go_on) {next}		

		chomp ($read_1[0],$read_1[1]);
		if (substr ($read_1[0], -1) =~ /\W/) {chop $read_1[0]; chop $read_1[1]}  # removes hex(0D) / CR from Unix/Linux
		$read_1[3] = length $read_1[0];   # one could check that length of sequence and quality is the same.
		$read_1[4] = "\t\n";

		chomp ($read_2[0],$read_2[1]);
		if (substr ($read_2[0], -1) =~ /\W/) {chop $read_2[0]; chop $read_2[1]}  # removes hex(0D) / CR from Unix/Linux
		$read_2[3] = length $read_2[0];   # one could check that length of sequence and quality is the same.
		$read_2[4] = "\t\n";
		
		$rd_out_1 = $rd_out_2 = readno_srt;
		if ( $qlength > 0 ) {	
			@qual_1 = split("",$read_1[1]);
			@qual_2 = split("",$read_2[1]);
			$qual_1 = $qual_2 = 0;
			for ($i = 0; $i < $qlength; $i++ ) {
				$qual_1 = $qual_1 + ord($qual_1[$i]) - $qbase;
				$qual_2 = $qual_2 + ord($qual_2[$i]) - $qbase;
			}
			$qual_1 = $qual_1 / $qlength;
			$qual_2 = $qual_2 / $qlength;
	
			if ($qual_1 < $qlimit) { 																 # Filter away criteria
				$rd_out_1 = read_lq_out;
			}					
			if ($qual_2 < $qlimit) { 																 # Filter away criteria
				$rd_out_2 = read_lq_out;
			}			
		}	# End if $qlength > 0
		
		$seqread_1 = join ("	",@read_1);
		print $rd_out_1 "$seqread_1";		
		$seqread_2 = join ("	",@read_2);
		print $rd_out_2 "$seqread_2";
		
		$r_no{$rd_out_1}++;
		$r_no{$rd_out_2}++;
		$r_no{total}++;
		$r_no{total_reads} = 2*$r_no{total};
				
		if (substr ($r_no{total_reads}, -6) == 0 ) {
			$rt_print = sprintf "%12s", sub_print_No ($r_no{total});
			$rs_print = sprintf "%12s", sub_print_No ($r_no{readno_srt});
			$rl_print = sprintf "%12s", sub_print_No ($r_no{read_lq_out});
			tid();
			print "$tid $rt_print paired reads; $rs_print passed; $rl_print low quality\n"
		}
	}	# End while go_on
	
	$rt_print = sprintf "%12s", sub_print_No ($r_no{total});
	$rs_print = sprintf "%12s", sub_print_No ($r_no{readno_srt});
	$rl_print = sprintf "%12s", sub_print_No ($r_no{read_lq_out});
	tid();
	print "$tid $rt_print paired reads; $rs_print passed; $rl_print low quality\n";

	read_check:
	if ($read_1[0] || $read_1[1] || $read_2[0] || $read_2[1] ){
		print "\nWarning: unequal no's of seq's  and qual's\n  Seq_1:  @read_1?\n  Seq_2:  @read_2?\n \n
		$seqpre_1
		$seqpre_2
		";
	}
	return ();
}  # sub read_fastq_PE

sub sub_print_No {
	my $No = $_[0];
	my $No_l = (length $No) -1;
	my $No_pr = "";
	my $t_3 = - int ($No_l / 3) - 1;
	while ( $t_3 < 0 ) {
		$No_pr .= substr ($No, 3*$t_3,3).",";
		$t_3++;
	}
	chop $No_pr;
	return($No_pr);
} # END sub sub_print_No

sub tid
{
	use Time::localtime;
	$tm = localtime;
	($Hr, $Min, $Sec) = ($tm -> hour, $tm -> min, $tm -> sec);
	$tid = sprintf("%02d:%02d:%02d",$Hr, $Min, $Sec);
	return()
} #end sub tid

sub Date
{
	use Time::localtime;
	$tm = localtime;
	($Day, $Month, $Year) = ($tm -> mday, $tm -> mon, $tm -> year);
	$date = sprintf("%02d-%02d-%02d",$Day, $Month+1, $Year+1900);
	return()
} #end sub Date

