#!/usr/bin/perl

# This script reformat reads in FASTA format to FASTQ quality base 64, sets all Q to "c" (40)

#
#    Copyright 2010, 2011 Ole Skovgaard.
#    The scripts are distributed under the terms of the GNU General Public License
#
#
#    This file is part of R2R - From Read to Results.
#
#    R2R is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, version 3 of the License.
#
#    R2R is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with R2R.  If not, see <http://www.gnu.org/licenses/>.
#
#    Version: Release_0.950, September, 2011
#

# use strict;
# use warnings;


my $screen = select (); # Saves the value for default output (to the screen), used after flushing of output files.


open(report, ">R2R_SolidFA_FastQ_Report.txt");

#  open(tst1, ">test1.txt");
#  open(tst2, ">test2.txt");
#  open(tst3, ">test3.txt");
#  select (tst1);    $| = 1;     # don't buffer print
#  select (tst2);    $| = 1;     # don't buffer print
#  select (tst3);    $| = 1;     # don't buffer print

select ($screen);


print "Use <File fasta> [<File fasta>]n  \n";

my ( $length, $argv, $fa, $r_l, $rn ) = "";
my ( %fq, %r_no ) = ();
my ( @read ) = ();

my $CR = chr(10);

open out,">out.fastq";
open discarded,">discarded.fa";

print "
The fasta sequences in the files: 
@ARGV 
were pseudo-formatted into FASTQ and saved in 'out.fastq'

";
print report "
The fasta sequences in the files: 
@ARGV 
were pseudo-formatted into FASTQ and saved in 'out.fastq'

";

foreach $argv (@ARGV) {
	my $tid = tid();
	print "Start:$tid \nSequence file: $argv \n";
	print report "Start:$tid \nSequence file: $argv \n";
	open(f_seq,"<$argv") or die "No Sequence file: $argv";

	while (<f_seq>) {
		chomp;
		$fa = $_;
		@read  ="";
		if ( $fa =~ />/ ) {next}
		if ( $fa =~ /[^acgt]/i ) {print discarded "$_"; $r_no{discarded}++; next }
		# unless ( $fa =~ /[acgt]/i ) {print discarded "$_"; $r_no{discarded}++; next }
		$r_no{total}++;

		$fa_length = length($fa);
		if ($fa_length > $fa_length_max) {$fa_length_max = $fa_length}
		$fq = "c"x$fa_length;
		$rn = sprintf("%08d",$r_no{total});
		$read[0] = "\@Read_$rn$CR";
		$read[1] = $fa.$CR;
		$read[2] = "+".$CR;
		$read[3] = $fq.$CR ;
		print out @read;
		$r_no{$fa_length}++;
	}

} # End foreach $argv

my $tid = tid();

$rn = sprintf("%8s",$r_no{total});
$rd = sprintf("%8s",$r_no{discarded});

print "
*********************************
End: $tid
Total number of reads:  \t$rn
Discarded reads:        \t$rd \n
";
print report "
*********************************
End: $tid
Total number of reads:  \t$rn
Discarded reads:        \t$rd \n
";
for $i (0 .. $fa_length_max) {
	if ($r_no{$i}) {
		$r_no{$i} = sprintf("%8s",$r_no{$i});
		$rii = sprintf("%4s",$i);
		print "Reads with length $rii: \t$r_no{$i}\n";
		print report "Reads with length $rii: \t$r_no{$i}\n";
	}
}


print "
*********************************
";

sub tid
{
	use Time::localtime;
	my $tm = localtime;
	my ($Hr, $Min, $Sec) = ($tm -> hour, $tm -> min, $tm -> sec);
	my $tid = sprintf("%02d:%02d:%02d",$Hr, $Min, $Sec);
	return($tid)
} #end sub tid

sub Date
{
	use Time::localtime;
	my $tm = localtime;
	my ($Day, $Month, $Year) = ($tm -> mday, $tm -> mon, $tm -> year);
	my $date = sprintf("%02d-%02d-%02d",$Day, $Month+1, $Year+1900);
	return()
} #end sub Date

