#!/usr/bin/perl
#		This script is to reduce complexity af transcription analysis (TA).
#		Use: R2R_TA.pl infile max_identical_reads   --fx: R2R_TA.pl foo_rd_srt.bex 3
#		TA will potentially yild an enormous amount of identical reads at selected positions.
#		This will be hard and meaningless to visualize.
#		The script reduces sequence-identical reads at each position in foo_rd_srt.bex to 3 and add a signature.
#		further there will be max 200 reads at all starting at each position (a fixed number set in the script).
#
#    Version: Release_0.910, September 20, 2010

$fid=$ARGV[0];
$max_identical_reads=$ARGV[1];

$max_read_at_pos = 200; # if more - it will be hard to visualize

unless (open(FA_seq,"<$fid"))  {
	print "Input file to be read analyzed \n" ;
	$fid=<stdin>;
	chomp $fid;
}
unless ($max_identical_reads)  {
	print "Maximum number of identical reads allowed?\n" ;
	$max_identical_reads=<stdin>;
	chomp $max_identical_reads;
}

close FA_seq;
print "\nReads in $fid will be analyzed \nMore than $max_identical_reads identical reads will be marked and removed\n" ;

open(FA_seq,"<$fid");
open(FAout,">$fid.TA");

$pos_prev = "";

while (<FA_seq>) {

	$read = $_;
	@rd = split "	",$read;
	$pos_ = $rd[0];

	unless ($pos_ == $pos_prev && @read) {
		@read_srt = @read [ sort {$read_idx[$a] cmp $read_idx[$b] } 0..@read ]; # Sorts  @read with @read_idx as index
		for $i (0 .. @read) {		# analyze all reads at this coordinate
			@rd_srt = split "	",$read_srt[$i];
			if ($rd_srt[4] eq $rd_srt_prev ) {		# read identical to prev. read
				$read_no++;

				if ($read_no == $max_identical_reads) {
					$read_substitute = $i+1;
					@rd_srt_w = @rd_srt;
				}

				if ($read_no > $max_identical_reads) {
					$read_srt[$i] = "" ;
				}

			}
			else {
				if ($read_no > $max_identical_reads) {
					$rd_srt_w[4] = $rd_srt_w[5] = "*" x length ($rd_srt[4]);
					$read_del = $read_no - $max_identical_reads;
					$warn = ( $rd_srt_w[2] eq "+"? " Warning: $read_no reads deleted " : revComp (" Warning: $read_no reads deleted "));
					substr ($rd_srt_w[4], 3, length $warn, $warn);
					$rd_srt_w[7] = "MMX:"; # $read_del
					$rd_srt_w[8] = "";
					$rd_srt_w[9] = "\n";
					$read_srt[$read_substitute] = join "\t", @rd_srt_w ;
				}
				$read_no = 1;
			}
		$rd_srt_prev = $rd_srt[4];
		}
		for $i (0 .. @read_srt) {		
			if ( $read_srt[$i]=~/\w/ ){  
				$read_srt[$i] =~ s/^\s+//;
				$read_out[$j]=$read_srt[$i]; 
				$j++;
				}
		}
		
		if ( @read_out>$max_read_at_pos) {
			$save = $read_out[$max_read_at_pos];
			$no = @read_out-$max_read_at_pos;		
			@read_out = splice (@read_out,0,$max_read_at_pos);
			@save = split "\t", $save;
			$save[4] = $save[5] = "#" x length ($save[4]);			
			$warn = ( $save[2] eq "+"? " Warning: $no reads deleted " : revComp (" Warning: $no reads deleted "));
			substr ($save[4], 3, length $warn, $warn);
			$save[7] = "MMX:"; 	# $no
			$save[8] = "";
			$save[9] = "\n";
			$read_out[$max_read_at_pos] = join "\t", @save;

		}
		
		print FAout " @read_out";
		if ($save[0] - $anal_prev > 1000 ) {print "Analyzed position: $save[0]\n"; $anal_prev=$save[0] }

		$read_no = 1;
		$j = 0;
		@read_idx = @read = @read_out = "";

	}

	push (@read,$read);			# push reads with same coordinates
	push (@read_idx,$rd[4]);		# and push their sequence for sorting

	$pos_prev = $pos_;
}
sub revComp
{
	my $r=reverse($_[0]);
	$r=~tr/ACGTacgtNn/TGCAtgcaNn/;
	return  $r;
} #revComp

